from dataclasses import dataclass
from typing import Optional

from _qwak_proto.qwak.feature_store.sources.batch_pb2 import (
    AnonymousS3Configuration as ProtoAnonymousS3Configuration,
    AwsS3AssumeRole as ProtoAwsS3AssumeRole,
    AwsS3FileSystemConfiguration as ProtoAwsS3FileSystemConfiguration,
    FileSystemConfiguration as ProtoFileSystemConfiguration,
)
from qwak.exceptions import QwakException
from qwak.feature_store.data_sources.batch.filesystem.base_config import (
    FileSystemConfiguration,
)


@dataclass
class AnonymousS3Configuration(FileSystemConfiguration):
    def _to_proto(self):
        return ProtoFileSystemConfiguration(
            aws_s3_anonymous=ProtoAnonymousS3Configuration()
        )

    @classmethod
    def _from_proto(cls, proto):
        return cls()


@dataclass
class AwsS3AssumeRoleFileSystemConfiguration(FileSystemConfiguration):
    role_arn: str

    def __post_init__(self):
        self._validate()

    def _validate(self):
        if not self.role_arn:
            raise QwakException("`role_arn` field is mandatory")

    def _to_proto(self) -> ProtoAwsS3FileSystemConfiguration:
        return ProtoFileSystemConfiguration(
            aws_s3_assume_role_configuration=ProtoAwsS3AssumeRole(
                role_arn=self.role_arn
            )
        )

    @classmethod
    def _from_proto(cls, proto: ProtoAwsS3AssumeRole):
        return cls(proto.role_arn)


@dataclass
class AwsS3FileSystemConfiguration(FileSystemConfiguration):
    access_key_secret_name: str
    secret_key_secret_name: str
    bucket: str
    session_token_secret_name: Optional[str] = ""

    def __post_init__(self):
        self._validate()

    def _validate(self):
        error_msg = "{field} field is mandatory"
        if not self.access_key_secret_name:
            raise QwakException(error_msg.format(field="access_key"))
        if not self.secret_key_secret_name:
            raise QwakException(error_msg.format(field="secret_key"))
        if not self.bucket:
            raise QwakException(error_msg.format(field="bucket"))

    def _to_proto(self):
        return ProtoFileSystemConfiguration(
            aws_s3_configuration=ProtoAwsS3FileSystemConfiguration(
                access_key_secret_name=self.access_key_secret_name,
                secret_key_secret_name=self.secret_key_secret_name,
                bucket=self.bucket,
                session_token_secret_name=self.session_token_secret_name,
            )
        )

    @classmethod
    def _from_proto(cls, proto):
        return AwsS3FileSystemConfiguration(
            access_key_secret_name=proto.access_key_secret_name,
            secret_key_secret_name=proto.secret_key_secret_name,
            bucket=proto.bucket,
            session_token_secret_name=proto.session_token_secret_name,
        )



from abc import ABC, abstractmethod
from typing import Optional, Tuple

from _qwak_proto.qwak.features_operator.v3.features_operator_async_service_pb2 import (
    GetValidationResultResponse as ProtoGetValidationResultResponse,
)
from qwak.clients.feature_store import FeatureRegistryClient
from qwak.clients.feature_store.operator_client import FeaturesOperatorClient
from qwak.exceptions import QwakException
from qwak.feature_store.data_sources.base import BaseSource
from qwak.feature_store.feature_sets.base_feature_set import BaseFeatureSet
from qwak.feature_store.validations.validation_decorators import (
    silence_backend_specific_validation_exceptions,
)
from qwak.feature_store.validations.validation_options import (
    DataSourceValidationOptions,
    FeatureSetValidationOptions,
)
from qwak.feature_store.validations.validation_response import (
    ValidationResponse,
    ValidationResponseFactory,
)


class Validator(ABC):
    @abstractmethod
    def validate_featureset(
        self,
        featureset: BaseFeatureSet,
        sample_size: int = 10,
        validation_options: Optional[FeatureSetValidationOptions] = None,
    ) -> Tuple[ValidationResponse, Optional[str]]:
        pass

    @abstractmethod
    def validate_data_source(
        self,
        data_source: BaseSource,
        sample_size: int = 10,
        validation_options: Optional[DataSourceValidationOptions] = None,
    ) -> Tuple[ValidationResponse, Optional[str]]:
        pass


class FeaturesOperatorValidator(Validator):
    _operator_client: FeaturesOperatorClient
    _registry_client: FeatureRegistryClient

    def __init__(self):
        try:
            import pandas  # noqa: F401
        except ImportError:
            raise QwakException("Missing required Pandas dependency")

        self._operator_client = FeaturesOperatorClient()
        self._registry_client = FeatureRegistryClient()

    @silence_backend_specific_validation_exceptions(
        "Validating DataSource is not supported for self-hosted environments"
    )
    def validate_data_source(
        self,
        data_source: BaseSource,
        sample_size: int = 10,
        validation_options: Optional[DataSourceValidationOptions] = None,
    ) -> Tuple[ValidationResponse, Optional[str]]:
        if sample_size <= 0 or 1_000 < sample_size:
            raise ValueError(
                f"sample_size must be under 1000 and positive, got: {sample_size}"
            )
        artifact_url: Optional[str] = None
        data_source_spec, artifact_url = data_source._prepare_and_get()
        proto_response: ProtoGetValidationResultResponse = (
            self._operator_client.validate_data_source_blocking(
                data_source_spec=data_source_spec,
                num_samples=sample_size,
                validation_options=(
                    validation_options.to_proto() if validation_options else None
                ),
            )
        )

        return ValidationResponseFactory.from_proto(proto_response), artifact_url

    @silence_backend_specific_validation_exceptions(
        "Validating FeatureSet is not supported for self-hosted environments"
    )
    def validate_featureset(
        self,
        featureset: BaseFeatureSet,
        sample_size: int = 10,
        validation_options: Optional[FeatureSetValidationOptions] = None,
    ) -> Tuple[ValidationResponse, Optional[str]]:
        if sample_size <= 0 or 1_000 < sample_size:
            raise ValueError(
                f"sample_size must be under 1000 and positive, got: {sample_size}"
            )
        artifact_url: Optional[str] = None
        featureset_spec, artifact_url = featureset._to_proto(
            feature_registry=self._registry_client, features=None, git_commit=None
        )

        proto_response: ProtoGetValidationResultResponse = (
            self._operator_client.validate_featureset_blocking(
                featureset_spec=featureset_spec,
                resource_path=artifact_url,
                num_samples=sample_size,
                validation_options=(
                    validation_options.to_proto() if validation_options else None
                ),
            )
        )

        return ValidationResponseFactory.from_proto(proto_response), artifact_url



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import qwak.admiral.secret.v0.secret_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class AccountSystemSecretIdentifier(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACCOUNT_ID_FIELD_NUMBER: builtins.int
    SECRET_NAME_FIELD_NUMBER: builtins.int
    account_id: builtins.str
    """account id"""
    secret_name: builtins.str
    """secret name"""
    def __init__(
        self,
        *,
        account_id: builtins.str = ...,
        secret_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["account_id", b"account_id", "secret_name", b"secret_name"]) -> None: ...

global___AccountSystemSecretIdentifier = AccountSystemSecretIdentifier

class AccountSystemSecretDefinition(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SPEC_FIELD_NUMBER: builtins.int
    @property
    def spec(self) -> global___AccountSystemSecretSpec:
        """Secret spec"""
    def __init__(
        self,
        *,
        spec: global___AccountSystemSecretSpec | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["spec", b"spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["spec", b"spec"]) -> None: ...

global___AccountSystemSecretDefinition = AccountSystemSecretDefinition

class AccountSystemSecretSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACCOUNT_IDENTIFIER_FIELD_NUMBER: builtins.int
    ACCOUNT_SYSTEM_SECRET_FIELD_NUMBER: builtins.int
    @property
    def account_identifier(self) -> global___AccountSystemSecretIdentifier:
        """Secret identifier"""
    @property
    def account_system_secret(self) -> qwak.admiral.secret.v0.secret_pb2.SystemSecretSpec:
        """Secret"""
    def __init__(
        self,
        *,
        account_identifier: global___AccountSystemSecretIdentifier | None = ...,
        account_system_secret: qwak.admiral.secret.v0.secret_pb2.SystemSecretSpec | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["account_identifier", b"account_identifier", "account_system_secret", b"account_system_secret"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["account_identifier", b"account_identifier", "account_system_secret", b"account_system_secret"]) -> None: ...

global___AccountSystemSecretSpec = AccountSystemSecretSpec



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _PurchaseOption:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _PurchaseOptionEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_PurchaseOption.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    INVALID_PURCHASE_OPTION: _PurchaseOption.ValueType  # 0
    SPOT_PURCHASE_OPTION: _PurchaseOption.ValueType  # 1
    ONDEMAND_PURCHASE_OPTION: _PurchaseOption.ValueType  # 2

class PurchaseOption(_PurchaseOption, metaclass=_PurchaseOptionEnumTypeWrapper): ...

INVALID_PURCHASE_OPTION: PurchaseOption.ValueType  # 0
SPOT_PURCHASE_OPTION: PurchaseOption.ValueType  # 1
ONDEMAND_PURCHASE_OPTION: PurchaseOption.ValueType  # 2
global___PurchaseOption = PurchaseOption



import os
from abc import ABC
from typing import List, Optional

from pydantic import BaseModel

from frogml_storage.utils import calc_content_sha2, calculate_sha2


class Checksums(BaseModel):
    sha2: str

    @classmethod
    def calc_checksums(cls, file_path: str) -> "Checksums":
        return cls(sha2=calculate_sha2(file_path))

    @classmethod
    def calc_content_checksums(cls, content: str) -> "Checksums":
        return cls(sha2=calc_content_sha2(content))


class Artifact(BaseModel):
    artifact_path: str
    size: int
    checksums: Checksums

    def __eq__(self, other):
        if not isinstance(other, Artifact):
            return False
        return (
            self.artifact_path == other.artifact_path
            and self.size == other.size
            and self.checksums == other.checksums
        )


class EntityManifest(BaseModel, ABC):
    """
    Represent an entity manifest file

    Attributes:
        created_date: The date the model | dataset were uploaded to Artifactory
        artifacts: A list of artifacts that belong to the model | dataset
        id: <organization>/<entity_name> - exists only for downloaded EntityInfo
        version: The entity version - exists only for downloaded EntityInfo
    """

    created_date: str
    artifacts: List[Artifact]
    id: Optional[str] = None
    version: Optional[str] = None

    def add_file(self, file_path: str, checksums: Checksums, rel_path: str) -> None:
        self.artifacts.append(
            Artifact(
                artifact_path=rel_path,
                size=os.path.getsize(file_path),
                checksums=checksums,
            )
        )

    def add_content_file(self, rel_path: str, content: str) -> None:
        checksums = Checksums.calc_content_checksums(content)
        self.artifacts.append(
            Artifact(
                artifact_path=rel_path,
                size=len((content, "utf-8")),
                checksums=checksums,
            )
        )

    @classmethod
    def from_json(cls, json_str: str) -> "EntityManifest":
        return cls.model_validate_json(json_str)

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_none=True)

    def __eq__(self, other):
        if not isinstance(other, EntityManifest):
            return False
        if self.id != other.id:
            return False
        if self.version != other.version:
            return False
        if self.created_date != other.created_date:
            return False
        if len(self.artifacts) != len(other.artifacts):
            return False
        for self_artifact, other_artifact in zip(self.artifacts, other.artifacts):
            if self_artifact != other_artifact:
                return False
        return True



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/projects/projects.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from _qwak_proto.qwak.models import models_pb2 as qwak_dot_models_dot_models__pb2
from _qwak_proto.qwak.projects import jfrog_project_spec_pb2 as qwak_dot_projects_dot_jfrog__project__spec__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1cqwak/projects/projects.proto\x12\x18qwak.projects.management\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x18qwak/models/models.proto\x1a&qwak/projects/jfrog_project_spec.proto\"\xad\x03\n\x0eProjectRequest\x12P\n\x16\x63reate_project_request\x18\x01 \x01(\x0b\x32..qwak.projects.management.CreateProjectRequestH\x00\x12P\n\x16\x64\x65lete_project_request\x18\x02 \x01(\x0b\x32..qwak.projects.management.DeleteProjectRequestH\x00\x12P\n\x16update_project_request\x18\x03 \x01(\x0b\x32..qwak.projects.management.UpdateProjectRequestH\x00\x12J\n\x13get_project_request\x18\x04 \x01(\x0b\x32+.qwak.projects.management.GetProjectRequestH\x00\x12N\n\x15list_projects_request\x18\x05 \x01(\x0b\x32-.qwak.projects.management.ListProjectsRequestH\x00\x42\t\n\x07request\"\x91\x01\n\x14\x43reateProjectRequest\x12\x14\n\x0cproject_name\x18\x01 \x01(\t\x12\x1b\n\x13project_description\x18\x02 \x01(\t\x12\x46\n\njfrog_spec\x18\x03 \x01(\x0b\x32\x32.qwak.projects.management.ModelRepositoryJFrogSpec\"O\n\x15\x43reateProjectResponse\x12\x36\n\x07project\x18\x01 \x01(\x0b\x32%.qwak.projects.management.ProjectSpec\"\x15\n\x13ListProjectsRequest\"O\n\x14ListProjectsResponse\x12\x37\n\x08projects\x18\x01 \x03(\x0b\x32%.qwak.projects.management.ProjectSpec\"=\n\x11GetProjectRequest\x12\x12\n\nproject_id\x18\x01 \x01(\t\x12\x14\n\x0cproject_name\x18\x02 \x01(\t\"H\n\x12GetProjectResponse\x12\x32\n\x07project\x18\x01 \x01(\x0b\x32!.qwak.projects.management.Project\"*\n\x14\x44\x65leteProjectRequest\x12\x12\n\nproject_id\x18\x01 \x01(\t\"%\n\x15\x44\x65leteProjectResponse\x12\x0c\n\x04info\x18\x01 \x01(\t\"{\n\x14UpdateProjectRequest\x12\x12\n\nproject_id\x18\x01 \x01(\t\x12O\n\x13project_spec_config\x18\x02 \x01(\x0b\x32\x32.qwak.projects.management.ProjectSpecConfiguration\"T\n\x15UpdateProjectResponse\x12;\n\x0cproject_spec\x18\x01 \x01(\x0b\x32%.qwak.projects.management.ProjectSpec\"\xac\x01\n\x07Project\x12\x33\n\x04spec\x18\x01 \x01(\x0b\x32%.qwak.projects.management.ProjectSpec\x12\x31\n\x06models\x18\x02 \x03(\x0b\x32\x1d.qwak.models.management.ModelB\x02\x18\x01\x12\x39\n\rmodels_briefs\x18\x03 \x03(\x0b\x32\".qwak.models.management.ModelBrief\"\xeb\x03\n\x0bProjectSpec\x12\x12\n\nproject_id\x18\x01 \x01(\t\x12\x14\n\x0cproject_name\x18\x02 \x01(\t\x12\x1b\n\x13project_description\x18\x03 \x01(\t\x12\x44\n\x0eproject_status\x18\x04 \x01(\x0e\x32,.qwak.projects.management.ProjectSpec.Status\x12.\n\ncreated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x12\n\ncreated_by\x18\x06 \x01(\t\x12\x34\n\x10last_modified_at\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x18\n\x10last_modified_by\x18\x08 \x01(\t\x12\x14\n\x0cmodels_count\x18\t \x01(\x05\x12\x15\n\rmodels_active\x18\n \x01(\x05\x12N\n\x0ejfrog_settings\x18\x0b \x01(\x0b\x32\x36.qwak.projects.management.ModelRepositoryJFrogSettings\">\n\x06Status\x12\x0b\n\x07INVALID\x10\x00\x12\x0c\n\x08\x44ISABLED\x10\x01\x12\r\n\tSUSPENDED\x10\x02\x12\n\n\x06\x41\x43TIVE\x10\x03\"Q\n\x18ProjectSpecConfiguration\x12\x18\n\x0cproject_name\x18\x01 \x01(\tB\x02\x18\x01\x12\x1b\n\x13project_description\x18\x02 \x01(\t\"\xb1\x02\n\x1cModelRepositoryJFrogSettings\x12S\n\x11pull_image_secret\x18\x01 \x01(\x0b\x32\x38.qwak.projects.management.ModelRepositoryPullImageSecret\x12[\n\x15jfrog_project_details\x18\x02 \x01(\x0b\x32<.qwak.projects.management.ModelRepositoryJFrogProjectDetails\x12_\n\x1ajfrog_repositories_details\x18\x03 \x01(\x0b\x32;.qwak.projects.management.ModelRepositoryArtifactoryDetails\"@\n\x1eModelRepositoryPullImageSecret\x12\x1e\n\x16pull_image_secret_name\x18\x01 \x01(\t\"?\n\"ModelRepositoryJFrogProjectDetails\x12\x19\n\x11jfrog_project_key\x18\x01 \x01(\t\"\xb7\x02\n!ModelRepositoryArtifactoryDetails\x12[\n\x1b\x61rtifact_repository_details\x18\x01 \x01(\x0b\x32\x36.qwak.projects.management.ArtifactoryRepositoryDetails\x12Z\n\x1a\x64\x61taset_repository_details\x18\x02 \x01(\x0b\x32\x36.qwak.projects.management.ArtifactoryRepositoryDetails\x12Y\n\x19\x64ocker_repository_details\x18\x03 \x01(\x0b\x32\x36.qwak.projects.management.ArtifactoryRepositoryDetails\"6\n\x1c\x41rtifactoryRepositoryDetails\x12\x16\n\x0erepository_key\x18\x01 \x01(\t2\xc9\x04\n\x19ProjectsManagementService\x12p\n\rCreateProject\x12..qwak.projects.management.CreateProjectRequest\x1a/.qwak.projects.management.CreateProjectResponse\x12m\n\x0cListProjects\x12-.qwak.projects.management.ListProjectsRequest\x1a..qwak.projects.management.ListProjectsResponse\x12g\n\nGetProject\x12+.qwak.projects.management.GetProjectRequest\x1a,.qwak.projects.management.GetProjectResponse\x12p\n\rDeleteProject\x12..qwak.projects.management.DeleteProjectRequest\x1a/.qwak.projects.management.DeleteProjectResponse\x12p\n\rUpdateProject\x12..qwak.projects.management.UpdateProjectRequest\x1a/.qwak.projects.management.UpdateProjectResponseB\'\n#com.qwak.ai.management.projects.apiP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.projects.projects_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n#com.qwak.ai.management.projects.apiP\001'
  _PROJECT.fields_by_name['models']._options = None
  _PROJECT.fields_by_name['models']._serialized_options = b'\030\001'
  _PROJECTSPECCONFIGURATION.fields_by_name['project_name']._options = None
  _PROJECTSPECCONFIGURATION.fields_by_name['project_name']._serialized_options = b'\030\001'
  _globals['_PROJECTREQUEST']._serialized_start=158
  _globals['_PROJECTREQUEST']._serialized_end=587
  _globals['_CREATEPROJECTREQUEST']._serialized_start=590
  _globals['_CREATEPROJECTREQUEST']._serialized_end=735
  _globals['_CREATEPROJECTRESPONSE']._serialized_start=737
  _globals['_CREATEPROJECTRESPONSE']._serialized_end=816
  _globals['_LISTPROJECTSREQUEST']._serialized_start=818
  _globals['_LISTPROJECTSREQUEST']._serialized_end=839
  _globals['_LISTPROJECTSRESPONSE']._serialized_start=841
  _globals['_LISTPROJECTSRESPONSE']._serialized_end=920
  _globals['_GETPROJECTREQUEST']._serialized_start=922
  _globals['_GETPROJECTREQUEST']._serialized_end=983
  _globals['_GETPROJECTRESPONSE']._serialized_start=985
  _globals['_GETPROJECTRESPONSE']._serialized_end=1057
  _globals['_DELETEPROJECTREQUEST']._serialized_start=1059
  _globals['_DELETEPROJECTREQUEST']._serialized_end=1101
  _globals['_DELETEPROJECTRESPONSE']._serialized_start=1103
  _globals['_DELETEPROJECTRESPONSE']._serialized_end=1140
  _globals['_UPDATEPROJECTREQUEST']._serialized_start=1142
  _globals['_UPDATEPROJECTREQUEST']._serialized_end=1265
  _globals['_UPDATEPROJECTRESPONSE']._serialized_start=1267
  _globals['_UPDATEPROJECTRESPONSE']._serialized_end=1351
  _globals['_PROJECT']._serialized_start=1354
  _globals['_PROJECT']._serialized_end=1526
  _globals['_PROJECTSPEC']._serialized_start=1529
  _globals['_PROJECTSPEC']._serialized_end=2020
  _globals['_PROJECTSPEC_STATUS']._serialized_start=1958
  _globals['_PROJECTSPEC_STATUS']._serialized_end=2020
  _globals['_PROJECTSPECCONFIGURATION']._serialized_start=2022
  _globals['_PROJECTSPECCONFIGURATION']._serialized_end=2103
  _globals['_MODELREPOSITORYJFROGSETTINGS']._serialized_start=2106
  _globals['_MODELREPOSITORYJFROGSETTINGS']._serialized_end=2411
  _globals['_MODELREPOSITORYPULLIMAGESECRET']._serialized_start=2413
  _globals['_MODELREPOSITORYPULLIMAGESECRET']._serialized_end=2477
  _globals['_MODELREPOSITORYJFROGPROJECTDETAILS']._serialized_start=2479
  _globals['_MODELREPOSITORYJFROGPROJECTDETAILS']._serialized_end=2542
  _globals['_MODELREPOSITORYARTIFACTORYDETAILS']._serialized_start=2545
  _globals['_MODELREPOSITORYARTIFACTORYDETAILS']._serialized_end=2856
  _globals['_ARTIFACTORYREPOSITORYDETAILS']._serialized_start=2858
  _globals['_ARTIFACTORYREPOSITORYDETAILS']._serialized_end=2912
  _globals['_PROJECTSMANAGEMENTSERVICE']._serialized_start=2915
  _globals['_PROJECTSMANAGEMENTSERVICE']._serialized_end=3500
# @@protoc_insertion_point(module_scope)



from dataclasses import dataclass
from datetime import datetime, timezone
from typing import List, Optional, Set, Union

from _qwak_proto.qwak.feature_store.features.execution_pb2 import (
    ExecutionSpec as ProtoExecutionSpec,
)
from _qwak_proto.qwak.execution.v1.streaming_aggregation_pb2 import (
    StreamingAggregationBackfillIngestion as ProtoStreamingAggregationBackfillIngestion,
    BackfillDataSource as ProtoBackfillDataSource,
    TimeRange as ProtoTimeRange,
)
from google.protobuf.timestamp_pb2 import Timestamp as ProtoTimestamp
from qwak.exceptions import QwakException
from qwak.feature_store._common.artifact_utils import ArtifactSpec, ArtifactsUploader
from qwak.feature_store.feature_sets.execution_spec import ClusterTemplate
from qwak.feature_store.feature_sets.transformations import SparkSqlTransformation

_BACKFILL_ = "_qwak_backfill_specification"


@dataclass
class BackfillDataSource:
    data_source_name: str
    start_datetime: Optional[datetime] = None
    end_datetime: Optional[datetime] = None

    def __post_init__(self):
        self._validate()

    def _to_proto(self) -> ProtoBackfillDataSource:
        start_timestamp: Optional[ProtoTimestamp] = None
        end_timestamp: Optional[ProtoTimestamp] = None

        if self.end_datetime:
            end_timestamp = ProtoTimestamp()
            end_timestamp.FromDatetime(self.end_datetime.astimezone(timezone.utc))

        if self.start_datetime:
            start_timestamp = ProtoTimestamp()
            start_timestamp.FromDatetime(self.start_datetime.astimezone(timezone.utc))

        time_range = ProtoTimeRange(
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
        )

        return ProtoBackfillDataSource(
            data_source_name=self.data_source_name,
            time_range=time_range,
        )

    @classmethod
    def _from_proto(cls, proto: ProtoBackfillDataSource) -> "BackfillDataSource":
        start_datetime: Optional[datetime] = None
        end_datetime: Optional[datetime] = None

        time_range: ProtoTimeRange = proto.time_range

        proto_start_timestamp: Optional[ProtoTimestamp] = (
            time_range.start_timestamp if time_range.start_timestamp else None
        )
        proto_end_timestamp: Optional[ProtoTimestamp] = (
            time_range.end_timestamp if time_range.end_timestamp else None
        )

        start_datetime = (
            datetime.fromtimestamp(
                proto_start_timestamp.seconds + proto_start_timestamp.nanos / 1e9
            )
            if proto_start_timestamp
            else None
        )

        end_datetime = (
            datetime.fromtimestamp(
                proto_end_timestamp.seconds + proto_end_timestamp.nanos / 1e9
            )
            if proto_end_timestamp
            else None
        )

        return cls(
            data_source_name=proto.data_source_name,
            start_datetime=start_datetime,
            end_datetime=end_datetime,
        )

    def _validate(self):
        if self.start_datetime and self.end_datetime:
            if self.start_datetime >= self.end_datetime:
                raise QwakException(
                    f"Backfill data source {self.data_source_name} has invalid time range: "
                    f"start_datetime {self.start_datetime} is after or equal end_datetime {self.end_datetime}."
                )

        if not self.data_source_name:
            raise QwakException(
                "Backfill data source must have a valid data source name."
            )


@dataclass
class StreamingBackfill:
    featureset_name: str
    start_datetime: datetime
    end_datetime: datetime
    data_sources: List[BackfillDataSource]
    transform: "SparkSqlTransformation"
    cluster_template: Optional[ClusterTemplate] = ClusterTemplate.SMALL

    def __post_init__(self):
        if not self.featureset_name:
            raise QwakException("featureset_name must be provided for backfill.")

        if not self.start_datetime or not self.end_datetime:
            raise QwakException(
                "For Streaming backfill, start_datetime and end_datetime are mandatory fields."
            )

        if self.start_datetime >= self.end_datetime:
            raise QwakException(
                f"Backfill has invalid time range: "
                f"start_datetime {self.start_datetime} is after or equal end_datetime {self.end_datetime}."
            )

        if not self.data_sources:
            raise QwakException(
                "Trying to create a streaming backfill with no data sources. "
                "At least one data source has to be provided when trying to create a streaming backfill."
            )

        if type(self.transform) is not SparkSqlTransformation:
            raise QwakException(
                "For backfill, only Spark SQL transformation type is currently supported"
            )

        self._validate_unique_sources()

    def _validate_unique_sources(self):
        source_names: List[str] = [
            data_source.data_source_name for data_source in self.data_sources
        ]
        duplicates: Set[str] = {
            item for item in source_names if source_names.count(item) > 1
        }
        if duplicates:
            raise QwakException(
                f"A specific data source can only appear once per backfill definition. "
                f"Found these duplicates: {', '.join(set(duplicates))}"
            )

    def _to_proto(
        self,
        original_instance_module_path: str,
    ) -> ProtoStreamingAggregationBackfillIngestion:
        artifact_url: Optional[str] = None
        artifact_spec: Optional[ArtifactSpec] = ArtifactsUploader.get_artifact_spec(
            transformation=self.transform,
            featureset_name=f"{self.featureset_name}-backfill",
            __instance_module_path__=original_instance_module_path,
        )

        if artifact_spec:
            artifact_url = ArtifactsUploader.upload(artifact_spec)

        end_timestamp = ProtoTimestamp()
        end_timestamp.FromDatetime(self.end_datetime.astimezone(timezone.utc))

        start_timestamp = ProtoTimestamp()
        start_timestamp.FromDatetime(self.start_datetime.astimezone(timezone.utc))

        return ProtoStreamingAggregationBackfillIngestion(
            featureset_name=self.featureset_name,
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
            execution_spec=ProtoExecutionSpec(
                cluster_template=ClusterTemplate.to_proto(self.cluster_template)
            ),
            transformation=self.transform._to_proto(artifact_path=artifact_url),
            data_source_specs=[
                data_source._to_proto() for data_source in self.data_sources
            ],
        )

    @classmethod
    def _from_proto(cls, proto: ProtoStreamingAggregationBackfillIngestion):
        backfill_data_sources = [
            BackfillDataSource._from_proto(ds) for ds in proto.data_source_specs
        ]

        return cls(
            featureset_name=proto.featureset_name,
            start_datetime=datetime.fromtimestamp(
                proto.start_timestamp.seconds + proto.start_timestamp.nanos / 1e9
            ),
            end_datetime=datetime.fromtimestamp(
                proto.end_timestamp.seconds + proto.end_timestamp.nanos / 1e9
            ),
            data_sources=backfill_data_sources,
            transform=SparkSqlTransformation._from_proto(
                proto.transformation.sql_transformation
            ),
            cluster_template=(
                ClusterTemplate.from_proto(proto.execution_spec.cluster_template)
                if proto.execution_spec.cluster_template
                else None
            ),
        )

    @staticmethod
    def _get_normalized_backfill_sources_spec(
        data_sources: Union[List[str], List[BackfillDataSource]],
    ) -> List[BackfillDataSource]:
        # reformat all data source names to 'BackfillDataSource'
        return [
            (
                BackfillDataSource(data_source_name=data_source)
                if isinstance(data_source, str)
                else data_source
            )
            for data_source in data_sources
        ]



from dataclasses import dataclass, field
from pathlib import Path
from typing import TYPE_CHECKING, Any, Callable, List, Optional, Union

from qwak.clients.feature_store.management_client import FeatureRegistryClient
from qwak.feature_store._common.packaging import upload_artifact

if TYPE_CHECKING:
    from qwak.feature_store.data_sources.streaming.kafka.deserialization import (
        Deserializer,
    )
    from qwak.feature_store.feature_sets.transformations.transformations import (
        BaseTransformation,
    )


@dataclass
class ArtifactSpec:
    """
    Dataclass for holding artifacts which will be uploaded
    """

    artifact_name: str
    root_module_path: Path
    artifact_object: Union["BaseTransformation", "Deserializer"]
    callables: List[Callable] = field(default_factory=list)
    suffix: str = ""


class ArtifactsUploader:
    @staticmethod
    def upload(artifact: ArtifactSpec) -> str:
        return upload_artifact(
            feature_store_object_name=artifact.artifact_name,
            feature_store_object_name_suffix=artifact.suffix,
            functions=artifact.callables,
            feature_module_dir=artifact.root_module_path,
            features_manager_client=FeatureRegistryClient(),
            artifact_object=artifact.artifact_object,
        )

    @staticmethod
    def get_artifact_spec(
        transformation: "BaseTransformation",
        featureset_name: str,
        __instance_module_path__: str,
    ) -> Optional[ArtifactSpec]:
        transformation_functions: Optional[List[Callable[..., Any]]] = (
            transformation.get_functions()
        )
        if not transformation_functions:
            return None

        return ArtifactSpec(
            artifact_name=featureset_name,
            root_module_path=Path(__instance_module_path__).parent,
            artifact_object=transformation,
            callables=transformation_functions,
        )



PIP_DEPENDENCY_FILE = "requirements.txt"
CONDA_YML_DEPENDENCY_FILE = "conda.yml"
CONDA_YAML_DEPENDENCY_FILE = "conda.yaml"
POETRY_DEPENDENCY_FILE = "pyproject.toml"
POETRY_LOCK_DEPENDENCY_FILE = "poetry.lock"



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import qwak.prompt.v1.prompt.prompt_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class CreatePromptRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROMPT_NAME_FIELD_NUMBER: builtins.int
    PROMPT_VERSION_SPEC_FIELD_NUMBER: builtins.int
    PROMPT_SPEC_FIELD_NUMBER: builtins.int
    prompt_name: builtins.str
    """Prompt name"""
    @property
    def prompt_version_spec(self) -> qwak.prompt.v1.prompt.prompt_pb2.PromptVersionSpec:
        """Prompt Version spec"""
    @property
    def prompt_spec(self) -> qwak.prompt.v1.prompt.prompt_pb2.PromptSpec:
        """Prompt spec"""
    def __init__(
        self,
        *,
        prompt_name: builtins.str = ...,
        prompt_version_spec: qwak.prompt.v1.prompt.prompt_pb2.PromptVersionSpec | None = ...,
        prompt_spec: qwak.prompt.v1.prompt.prompt_pb2.PromptSpec | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["prompt_spec", b"prompt_spec", "prompt_version_spec", b"prompt_version_spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["prompt_name", b"prompt_name", "prompt_spec", b"prompt_spec", "prompt_version_spec", b"prompt_version_spec"]) -> None: ...

global___CreatePromptRequest = CreatePromptRequest

class CreatePromptResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROMPT_FIELD_NUMBER: builtins.int
    @property
    def prompt(self) -> qwak.prompt.v1.prompt.prompt_pb2.Prompt:
        """Prompt definition"""
    def __init__(
        self,
        *,
        prompt: qwak.prompt.v1.prompt.prompt_pb2.Prompt | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["prompt", b"prompt"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["prompt", b"prompt"]) -> None: ...

global___CreatePromptResponse = CreatePromptResponse

class CreatePromptVersionRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROMPT_NAME_FIELD_NUMBER: builtins.int
    PROMPT_VERSION_SPEC_FIELD_NUMBER: builtins.int
    SET_DEFAULT_FIELD_NUMBER: builtins.int
    prompt_name: builtins.str
    """Prompt name"""
    @property
    def prompt_version_spec(self) -> qwak.prompt.v1.prompt.prompt_pb2.PromptVersionSpec:
        """Prompt version spec"""
    set_default: builtins.bool
    """Set this version as default"""
    def __init__(
        self,
        *,
        prompt_name: builtins.str = ...,
        prompt_version_spec: qwak.prompt.v1.prompt.prompt_pb2.PromptVersionSpec | None = ...,
        set_default: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["prompt_version_spec", b"prompt_version_spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["prompt_name", b"prompt_name", "prompt_version_spec", b"prompt_version_spec", "set_default", b"set_default"]) -> None: ...

global___CreatePromptVersionRequest = CreatePromptVersionRequest

class CreatePromptVersionResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROMPT_VERSION_FIELD_NUMBER: builtins.int
    @property
    def prompt_version(self) -> qwak.prompt.v1.prompt.prompt_pb2.PromptVersion:
        """Prompt version definition"""
    def __init__(
        self,
        *,
        prompt_version: qwak.prompt.v1.prompt.prompt_pb2.PromptVersion | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["prompt_version", b"prompt_version"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["prompt_version", b"prompt_version"]) -> None: ...

global___CreatePromptVersionResponse = CreatePromptVersionResponse

class UpdatePromptRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROMPT_NAME_FIELD_NUMBER: builtins.int
    PROMPT_SPEC_FIELD_NUMBER: builtins.int
    prompt_name: builtins.str
    """Prompt name"""
    @property
    def prompt_spec(self) -> qwak.prompt.v1.prompt.prompt_pb2.PromptSpec:
        """Prompt spec"""
    def __init__(
        self,
        *,
        prompt_name: builtins.str = ...,
        prompt_spec: qwak.prompt.v1.prompt.prompt_pb2.PromptSpec | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["prompt_spec", b"prompt_spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["prompt_name", b"prompt_name", "prompt_spec", b"prompt_spec"]) -> None: ...

global___UpdatePromptRequest = UpdatePromptRequest

class UpdatePromptResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___UpdatePromptResponse = UpdatePromptResponse

class GetPromptVersionByPromptNameRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROMPT_NAME_FIELD_NUMBER: builtins.int
    VERSION_NUMBER_FIELD_NUMBER: builtins.int
    prompt_name: builtins.str
    """Prompt name"""
    version_number: builtins.int
    def __init__(
        self,
        *,
        prompt_name: builtins.str = ...,
        version_number: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["optional_version_number", b"optional_version_number", "version_number", b"version_number"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["optional_version_number", b"optional_version_number", "prompt_name", b"prompt_name", "version_number", b"version_number"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["optional_version_number", b"optional_version_number"]) -> typing_extensions.Literal["version_number"] | None: ...

global___GetPromptVersionByPromptNameRequest = GetPromptVersionByPromptNameRequest

class GetPromptVersionByPromptNameResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROMPT_VERSION_FIELD_NUMBER: builtins.int
    @property
    def prompt_version(self) -> qwak.prompt.v1.prompt.prompt_pb2.PromptVersion:
        """Prompt version definition"""
    def __init__(
        self,
        *,
        prompt_version: qwak.prompt.v1.prompt.prompt_pb2.PromptVersion | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["prompt_version", b"prompt_version"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["prompt_version", b"prompt_version"]) -> None: ...

global___GetPromptVersionByPromptNameResponse = GetPromptVersionByPromptNameResponse

class GetPromptByNameRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROMPT_NAME_FIELD_NUMBER: builtins.int
    prompt_name: builtins.str
    """Prompt name"""
    def __init__(
        self,
        *,
        prompt_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["prompt_name", b"prompt_name"]) -> None: ...

global___GetPromptByNameRequest = GetPromptByNameRequest

class GetPromptByNameResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROMPT_FIELD_NUMBER: builtins.int
    @property
    def prompt(self) -> qwak.prompt.v1.prompt.prompt_pb2.Prompt:
        """Prompt"""
    def __init__(
        self,
        *,
        prompt: qwak.prompt.v1.prompt.prompt_pb2.Prompt | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["prompt", b"prompt"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["prompt", b"prompt"]) -> None: ...

global___GetPromptByNameResponse = GetPromptByNameResponse

class ListPromptsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ListPromptsRequest = ListPromptsRequest

class ListPromptsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROMPTS_FIELD_NUMBER: builtins.int
    @property
    def prompts(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[qwak.prompt.v1.prompt.prompt_pb2.Prompt]:
        """Prompts"""
    def __init__(
        self,
        *,
        prompts: collections.abc.Iterable[qwak.prompt.v1.prompt.prompt_pb2.Prompt] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["prompts", b"prompts"]) -> None: ...

global___ListPromptsResponse = ListPromptsResponse

class InitPaginationByPromptNameRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROMPT_NAME_FIELD_NUMBER: builtins.int
    prompt_name: builtins.str
    """prompt name"""
    def __init__(
        self,
        *,
        prompt_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["prompt_name", b"prompt_name"]) -> None: ...

global___InitPaginationByPromptNameRequest = InitPaginationByPromptNameRequest

class InitPaginationByPromptNameResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RECORD_COUNT_FIELD_NUMBER: builtins.int
    MAX_VERSION_NUMBER_FIELD_NUMBER: builtins.int
    record_count: builtins.int
    """Number of versions found for this prompt at the time of querying.
    this is used mainly by the client to plan things like number of pages etc.
    """
    max_version_number: builtins.int
    """the current highest version number for this prompt. this is the anchor
    to use on subsequent requests if the caller wants to achieve
    a view of a static list
    """
    def __init__(
        self,
        *,
        record_count: builtins.int = ...,
        max_version_number: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["max_version_number", b"max_version_number", "record_count", b"record_count"]) -> None: ...

global___InitPaginationByPromptNameResponse = InitPaginationByPromptNameResponse

class ListPromptVersionsByPromptNameRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROMPT_NAME_FIELD_NUMBER: builtins.int
    PAGE_SIZE_FIELD_NUMBER: builtins.int
    PAGE_NUMBER_FIELD_NUMBER: builtins.int
    MAX_VERSION_NUMBER_FIELD_NUMBER: builtins.int
    prompt_name: builtins.str
    """Prompt name"""
    page_size: builtins.int
    """The page size to use for the response - the number of results in the response
    is the minimum of the page_size and number of matching rows.
    - a negative value means all results will be returned (in that case page_number is ignored)
    """
    page_number: builtins.int
    """The page number to use"""
    max_version_number: builtins.int
    """The max version number to use as anchor - this is the value returned from InitPagination"""
    def __init__(
        self,
        *,
        prompt_name: builtins.str = ...,
        page_size: builtins.int = ...,
        page_number: builtins.int = ...,
        max_version_number: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["max_version_number", b"max_version_number", "page_number", b"page_number", "page_size", b"page_size", "prompt_name", b"prompt_name"]) -> None: ...

global___ListPromptVersionsByPromptNameRequest = ListPromptVersionsByPromptNameRequest

class ListPromptVersionsByPromptNameResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROMPT_VERSIONS_FIELD_NUMBER: builtins.int
    @property
    def prompt_versions(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[qwak.prompt.v1.prompt.prompt_pb2.PromptVersion]:
        """Prompt versions"""
    def __init__(
        self,
        *,
        prompt_versions: collections.abc.Iterable[qwak.prompt.v1.prompt.prompt_pb2.PromptVersion] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["prompt_versions", b"prompt_versions"]) -> None: ...

global___ListPromptVersionsByPromptNameResponse = ListPromptVersionsByPromptNameResponse

class DeletePromptRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROMPT_NAME_FIELD_NUMBER: builtins.int
    prompt_name: builtins.str
    """Prompt name"""
    def __init__(
        self,
        *,
        prompt_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["prompt_name", b"prompt_name"]) -> None: ...

global___DeletePromptRequest = DeletePromptRequest

class DeletePromptResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___DeletePromptResponse = DeletePromptResponse

class DeletePromptVersionRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROMPT_NAME_FIELD_NUMBER: builtins.int
    VERSION_NUMBER_FIELD_NUMBER: builtins.int
    prompt_name: builtins.str
    """Prompt name"""
    version_number: builtins.int
    """Prompt version"""
    def __init__(
        self,
        *,
        prompt_name: builtins.str = ...,
        version_number: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["prompt_name", b"prompt_name", "version_number", b"version_number"]) -> None: ...

global___DeletePromptVersionRequest = DeletePromptVersionRequest

class DeletePromptVersionResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___DeletePromptVersionResponse = DeletePromptVersionResponse

class SetDefaultPromptVersionRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROMPT_NAME_FIELD_NUMBER: builtins.int
    VERSION_NUMBER_FIELD_NUMBER: builtins.int
    prompt_name: builtins.str
    """Prompt name"""
    version_number: builtins.int
    """Version number"""
    def __init__(
        self,
        *,
        prompt_name: builtins.str = ...,
        version_number: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["prompt_name", b"prompt_name", "version_number", b"version_number"]) -> None: ...

global___SetDefaultPromptVersionRequest = SetDefaultPromptVersionRequest

class SetDefaultPromptVersionResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___SetDefaultPromptVersionResponse = SetDefaultPromptVersionResponse

class ListPromptBriefsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ListPromptBriefsRequest = ListPromptBriefsRequest

class ListPromptBriefsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROMPT_BRIEFS_FIELD_NUMBER: builtins.int
    @property
    def prompt_briefs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[qwak.prompt.v1.prompt.prompt_pb2.PromptBrief]:
        """Prompt briefs"""
    def __init__(
        self,
        *,
        prompt_briefs: collections.abc.Iterable[qwak.prompt.v1.prompt.prompt_pb2.PromptBrief] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["prompt_briefs", b"prompt_briefs"]) -> None: ...

global___ListPromptBriefsResponse = ListPromptBriefsResponse



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/instance_template/instance_template.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.user_application.common.v0 import resources_pb2 as qwak_dot_user__application_dot_common_dot_v0_dot_resources__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.qwak/instance_template/instance_template.proto\x12!qwak.management.instance_template\x1a/qwak/user_application/common/v0/resources.proto\"\xc9\x02\n\x14InstanceTemplateSpec\x12\n\n\x02id\x18\x01 \x01(\t\x12\x14\n\x0c\x64isplay_name\x18\x02 \x01(\t\x12\r\n\x05order\x18\x03 \x01(\x05\x12\x0f\n\x07\x65nabled\x18\x04 \x01(\x08\x12S\n\x15pod_compute_resources\x18\x05 \x01(\x0b\x32\x34.qwak.user_application.common.v0.PodComputeResources\x12\x46\n\rinstance_type\x18\x06 \x01(\x0e\x32/.qwak.management.instance_template.InstanceType\x12\x0b\n\x03qpu\x18\x07 \x01(\x02\x12\x15\n\raws_supported\x18\x08 \x01(\x08\x12\x15\n\rgcp_supported\x18\t \x01(\x08\x12\x17\n\x0f\x61zure_supported\x18\n \x01(\x08\"e\n\x0eInstanceFilter\x12S\n\x14instance_type_filter\x18\x01 \x01(\x0e\x32\x35.qwak.management.instance_template.InstanceTypeFilter*\x94\x01\n\x12InstanceTypeFilter\x12$\n INSTANCE_TYPE_FILTER_UNSPECIFIED\x10\x00\x12\x1c\n\x18INSTANCE_TYPE_FILTER_ALL\x10\x01\x12\x1c\n\x18INSTANCE_TYPE_FILTER_CPU\x10\x02\x12\x1c\n\x18INSTANCE_TYPE_FILTER_GPU\x10\x03*[\n\x0cInstanceType\x12\x1d\n\x19INSTANCE_TYPE_UNSPECIFIED\x10\x00\x12\x15\n\x11INSTANCE_TYPE_CPU\x10\x01\x12\x15\n\x11INSTANCE_TYPE_GPU\x10\x02\x42\x30\n,com.qwak.ai.management.instance_template.apiP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.instance_template.instance_template_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n,com.qwak.ai.management.instance_template.apiP\001'
  _globals['_INSTANCETYPEFILTER']._serialized_start=570
  _globals['_INSTANCETYPEFILTER']._serialized_end=718
  _globals['_INSTANCETYPE']._serialized_start=720
  _globals['_INSTANCETYPE']._serialized_end=811
  _globals['_INSTANCETEMPLATESPEC']._serialized_start=135
  _globals['_INSTANCETEMPLATESPEC']._serialized_end=464
  _globals['_INSTANCEFILTER']._serialized_start=466
  _globals['_INSTANCEFILTER']._serialized_end=567
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/admiral/secret/v0/system_secret_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.admiral.secret.v0 import secret_pb2 as qwak_dot_admiral_dot_secret_dot_v0_dot_secret__pb2
from _qwak_proto.qwak.admiral.secret.v0 import account_secret_pb2 as qwak_dot_admiral_dot_secret_dot_v0_dot_account__secret__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2qwak/admiral/secret/v0/system_secret_service.proto\x12\x16qwak.admiral.secret.v0\x1a#qwak/admiral/secret/v0/secret.proto\x1a+qwak/admiral/secret/v0/account_secret.proto\"d\n\x16SetSystemSecretRequest\x12J\n\x07options\x18\x01 \x01(\x0b\x32\x39.qwak.admiral.secret.v0.SetSystemEnvironmentSecretOptions\"\x19\n\x17SetSystemSecretResponse\"d\n\x19\x44\x65leteSystemSecretRequest\x12G\n\nidentifier\x18\x01 \x01(\x0b\x32\x33.qwak.admiral.secret.v0.EnvironmentSecretIdentifier\"\x1c\n\x1a\x44\x65leteSystemSecretResponse\"a\n\x16GetSystemSecretRequest\x12G\n\nidentifier\x18\x01 \x01(\x0b\x32\x33.qwak.admiral.secret.v0.EnvironmentSecretIdentifier\"d\n\x17GetSystemSecretResponse\x12I\n\x11secret_definition\x18\x01 \x01(\x0b\x32..qwak.admiral.secret.v0.SystemSecretDefinition\"^\n\x1dSetAccountSystemSecretRequest\x12=\n\x04spec\x18\x01 \x01(\x0b\x32/.qwak.admiral.secret.v0.AccountSystemSecretSpec\" \n\x1eSetAccountSystemSecretResponse\"r\n\x1dGetAccountSystemSecretRequest\x12Q\n\x12\x61\x63\x63ount_identifier\x18\x01 \x01(\x0b\x32\x35.qwak.admiral.secret.v0.AccountSystemSecretIdentifier\"z\n\x1eGetAccountSystemSecretResponse\x12X\n\x19\x61\x63\x63ount_secret_definition\x18\x01 \x01(\x0b\x32\x35.qwak.admiral.secret.v0.AccountSystemSecretDefinition\"u\n DeleteAccountSystemSecretRequest\x12Q\n\x12\x61\x63\x63ount_identifier\x18\x01 \x01(\x0b\x32\x35.qwak.admiral.secret.v0.AccountSystemSecretIdentifier\"#\n!DeleteAccountSystemSecretResponse2\xa1\x06\n\x13SystemSecretService\x12r\n\x0fSetSystemSecret\x12..qwak.admiral.secret.v0.SetSystemSecretRequest\x1a/.qwak.admiral.secret.v0.SetSystemSecretResponse\x12{\n\x12\x44\x65leteSystemSecret\x12\x31.qwak.admiral.secret.v0.DeleteSystemSecretRequest\x1a\x32.qwak.admiral.secret.v0.DeleteSystemSecretResponse\x12r\n\x0fGetSystemSecret\x12..qwak.admiral.secret.v0.GetSystemSecretRequest\x1a/.qwak.admiral.secret.v0.GetSystemSecretResponse\x12\x87\x01\n\x16SetAccountSystemSecret\x12\x35.qwak.admiral.secret.v0.SetAccountSystemSecretRequest\x1a\x36.qwak.admiral.secret.v0.SetAccountSystemSecretResponse\x12\x87\x01\n\x16GetAccountSystemSecret\x12\x35.qwak.admiral.secret.v0.GetAccountSystemSecretRequest\x1a\x36.qwak.admiral.secret.v0.GetAccountSystemSecretResponse\x12\x90\x01\n\x19\x44\x65leteAccountSystemSecret\x12\x38.qwak.admiral.secret.v0.DeleteAccountSystemSecretRequest\x1a\x39.qwak.admiral.secret.v0.DeleteAccountSystemSecretResponseB\x8c\x01\n\x1d\x63om.qwak.ai.admiral.secret.v0P\x01Zigithub.com/qwak-ai/qwak-platform/services/core/go/admiral/admiral-api/pb/qwak/admiral/secret/v0;secret_v0b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.admiral.secret.v0.system_secret_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\035com.qwak.ai.admiral.secret.v0P\001Zigithub.com/qwak-ai/qwak-platform/services/core/go/admiral/admiral-api/pb/qwak/admiral/secret/v0;secret_v0'
  _globals['_SETSYSTEMSECRETREQUEST']._serialized_start=160
  _globals['_SETSYSTEMSECRETREQUEST']._serialized_end=260
  _globals['_SETSYSTEMSECRETRESPONSE']._serialized_start=262
  _globals['_SETSYSTEMSECRETRESPONSE']._serialized_end=287
  _globals['_DELETESYSTEMSECRETREQUEST']._serialized_start=289
  _globals['_DELETESYSTEMSECRETREQUEST']._serialized_end=389
  _globals['_DELETESYSTEMSECRETRESPONSE']._serialized_start=391
  _globals['_DELETESYSTEMSECRETRESPONSE']._serialized_end=419
  _globals['_GETSYSTEMSECRETREQUEST']._serialized_start=421
  _globals['_GETSYSTEMSECRETREQUEST']._serialized_end=518
  _globals['_GETSYSTEMSECRETRESPONSE']._serialized_start=520
  _globals['_GETSYSTEMSECRETRESPONSE']._serialized_end=620
  _globals['_SETACCOUNTSYSTEMSECRETREQUEST']._serialized_start=622
  _globals['_SETACCOUNTSYSTEMSECRETREQUEST']._serialized_end=716
  _globals['_SETACCOUNTSYSTEMSECRETRESPONSE']._serialized_start=718
  _globals['_SETACCOUNTSYSTEMSECRETRESPONSE']._serialized_end=750
  _globals['_GETACCOUNTSYSTEMSECRETREQUEST']._serialized_start=752
  _globals['_GETACCOUNTSYSTEMSECRETREQUEST']._serialized_end=866
  _globals['_GETACCOUNTSYSTEMSECRETRESPONSE']._serialized_start=868
  _globals['_GETACCOUNTSYSTEMSECRETRESPONSE']._serialized_end=990
  _globals['_DELETEACCOUNTSYSTEMSECRETREQUEST']._serialized_start=992
  _globals['_DELETEACCOUNTSYSTEMSECRETREQUEST']._serialized_end=1109
  _globals['_DELETEACCOUNTSYSTEMSECRETRESPONSE']._serialized_start=1111
  _globals['_DELETEACCOUNTSYSTEMSECRETRESPONSE']._serialized_end=1146
  _globals['_SYSTEMSECRETSERVICE']._serialized_start=1149
  _globals['_SYSTEMSECRETSERVICE']._serialized_end=1950
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/administration/cluster/v2/cluster_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.administration.cluster.v2 import cluster_pb2 as qwak_dot_administration_dot_cluster_dot_v2_dot_cluster__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n4qwak/administration/cluster/v2/cluster_service.proto\x12\x1eqwak.administration.cluster.v2\x1a,qwak/administration/cluster/v2/cluster.proto\"h\n#CreateClusterCreationRequestRequest\x12\x41\n\x0c\x63luster_spec\x18\x01 \x01(\x0b\x32+.qwak.administration.cluster.v2.ClusterSpec\":\n$CreateClusterCreationRequestResponse\x12\x12\n\nrequest_id\x18\x01 \x01(\t\",\n\x16GetClusterStateRequest\x12\x12\n\nrequest_id\x18\x01 \x01(\t\"^\n\x17GetClusterStateResponse\x12\x43\n\rcluster_state\x18\x01 \x01(\x0b\x32,.qwak.administration.cluster.v2.ClusterState\"\'\n\x11GetClusterRequest\x12\x12\n\ncluster_id\x18\x01 \x01(\t\"Y\n\x12GetClusterResponse\x12\x43\n\rcluster_state\x18\x01 \x01(\x0b\x32,.qwak.administration.cluster.v2.ClusterState\"z\n!UpdateClusterConfigurationRequest\x12\x12\n\ncluster_id\x18\x01 \x01(\t\x12\x41\n\x0c\x63luster_spec\x18\x02 \x01(\x0b\x32+.qwak.administration.cluster.v2.ClusterSpec\"8\n\"UpdateClusterConfigurationResponse\x12\x12\n\nrequest_id\x18\x01 \x01(\t\"*\n\x14\x44\x65leteClusterRequest\x12\x12\n\ncluster_id\x18\x01 \x01(\t\"+\n\x15\x44\x65leteClusterResponse\x12\x12\n\nrequest_id\x18\x01 \x01(\t\"\x15\n\x13ListClustersRequest\"P\n\x14ListClustersResponse\x12\x38\n\x07\x63luster\x18\x01 \x03(\x0b\x32\'.qwak.administration.cluster.v2.Cluster2\xd5\x06\n\x0e\x43lusterService\x12\xa9\x01\n\x1c\x43reateClusterCreationRequest\x12\x43.qwak.administration.cluster.v2.CreateClusterCreationRequestRequest\x1a\x44.qwak.administration.cluster.v2.CreateClusterCreationRequestResponse\x12\x82\x01\n\x0fGetClusterState\x12\x36.qwak.administration.cluster.v2.GetClusterStateRequest\x1a\x37.qwak.administration.cluster.v2.GetClusterStateResponse\x12\xa3\x01\n\x1aUpdateClusterConfiguration\x12\x41.qwak.administration.cluster.v2.UpdateClusterConfigurationRequest\x1a\x42.qwak.administration.cluster.v2.UpdateClusterConfigurationResponse\x12|\n\rDeleteCluster\x12\x34.qwak.administration.cluster.v2.DeleteClusterRequest\x1a\x35.qwak.administration.cluster.v2.DeleteClusterResponse\x12s\n\nGetCluster\x12\x31.qwak.administration.cluster.v2.GetClusterRequest\x1a\x32.qwak.administration.cluster.v2.GetClusterResponse\x12y\n\x0cListClusters\x12\x33.qwak.administration.cluster.v2.ListClustersRequest\x1a\x34.qwak.administration.cluster.v2.ListClustersResponseB\xb4\x01\n)com.qwak.ai.administration.api.cluster.v2P\x01Z\x84\x01github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/administration/cluster/v2;cluster_v2b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.administration.cluster.v2.cluster_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n)com.qwak.ai.administration.api.cluster.v2P\001Z\204\001github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/administration/cluster/v2;cluster_v2'
  _globals['_CREATECLUSTERCREATIONREQUESTREQUEST']._serialized_start=134
  _globals['_CREATECLUSTERCREATIONREQUESTREQUEST']._serialized_end=238
  _globals['_CREATECLUSTERCREATIONREQUESTRESPONSE']._serialized_start=240
  _globals['_CREATECLUSTERCREATIONREQUESTRESPONSE']._serialized_end=298
  _globals['_GETCLUSTERSTATEREQUEST']._serialized_start=300
  _globals['_GETCLUSTERSTATEREQUEST']._serialized_end=344
  _globals['_GETCLUSTERSTATERESPONSE']._serialized_start=346
  _globals['_GETCLUSTERSTATERESPONSE']._serialized_end=440
  _globals['_GETCLUSTERREQUEST']._serialized_start=442
  _globals['_GETCLUSTERREQUEST']._serialized_end=481
  _globals['_GETCLUSTERRESPONSE']._serialized_start=483
  _globals['_GETCLUSTERRESPONSE']._serialized_end=572
  _globals['_UPDATECLUSTERCONFIGURATIONREQUEST']._serialized_start=574
  _globals['_UPDATECLUSTERCONFIGURATIONREQUEST']._serialized_end=696
  _globals['_UPDATECLUSTERCONFIGURATIONRESPONSE']._serialized_start=698
  _globals['_UPDATECLUSTERCONFIGURATIONRESPONSE']._serialized_end=754
  _globals['_DELETECLUSTERREQUEST']._serialized_start=756
  _globals['_DELETECLUSTERREQUEST']._serialized_end=798
  _globals['_DELETECLUSTERRESPONSE']._serialized_start=800
  _globals['_DELETECLUSTERRESPONSE']._serialized_end=843
  _globals['_LISTCLUSTERSREQUEST']._serialized_start=845
  _globals['_LISTCLUSTERSREQUEST']._serialized_end=866
  _globals['_LISTCLUSTERSRESPONSE']._serialized_start=868
  _globals['_LISTCLUSTERSRESPONSE']._serialized_end=948
  _globals['_CLUSTERSERVICE']._serialized_start=951
  _globals['_CLUSTERSERVICE']._serialized_end=1804
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/fitness_service/constructs.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%qwak/fitness_service/constructs.proto\x12\x14qwak.fitness.service\"\xa2\x01\n\x0f\x42uildProperties\x12\x10\n\x08\x62uild_id\x18\x01 \x01(\t\x12\x0e\n\x06\x62ranch\x18\x02 \x01(\t\x12\x10\n\x08model_id\x18\x03 \x01(\t\x12\x0c\n\x04tags\x18\x04 \x03(\t\x12\x35\n\tmodel_uri\x18\x05 \x01(\x0b\x32\".qwak.fitness.service.ModelUriSpec\x12\x16\n\x0egpu_compatible\x18\x06 \x01(\x08\"\x8d\x01\n\x0cModelUriSpec\x12\x0b\n\x03uri\x18\x01 \x01(\t\x12\x12\n\ngit_branch\x18\x02 \x01(\t\x12\x17\n\x0fgit_credentials\x18\x03 \x01(\t\x12\x1e\n\x16git_credentials_secret\x18\x04 \x01(\t\x12\x11\n\tcommit_id\x18\x05 \x01(\t\x12\x10\n\x08main_dir\x18\x06 \x01(\t\"t\n\x08\x42uildEnv\x12\x33\n\ndocker_env\x18\x01 \x01(\x0b\x32\x1f.qwak.fitness.service.DockerEnv\x12\x33\n\npython_env\x18\x02 \x01(\x0b\x32\x1f.qwak.fitness.service.PythonEnv\"\x8a\x01\n\tDockerEnv\x12\x12\n\nbase_image\x18\x01 \x01(\t\x12\x10\n\x08\x65nv_vars\x18\x02 \x03(\t\x12\x10\n\x08no_cache\x18\x03 \x01(\x08\x12\x1c\n\x14\x61ssumed_iam_role_arn\x18\x04 \x01(\t\x12\'\n\x1fservice_account_key_secret_name\x18\x05 \x01(\t\"|\n\tPythonEnv\x12\x17\n\x0fgit_credentials\x18\x01 \x01(\t\x12\x1e\n\x16git_credentials_secret\x18\x02 \x01(\t\x12\x18\n\x10qwak_sdk_version\x18\x03 \x01(\t\x12\x1c\n\x14qwak_sdk_extra_index\x18\x04 \x01(\t\"j\n\x0c\x43puResources\x12\x0b\n\x03\x63pu\x18\x01 \x01(\x02\x12\x15\n\rmemory_amount\x18\x02 \x01(\x05\x12\x36\n\x0cmemory_units\x18\x03 \x01(\x0e\x32 .qwak.fitness.service.MemoryUnit\"\xaf\x01\n\x0cGpuResources\x12/\n\x08gpu_type\x18\x01 \x01(\x0e\x32\x1d.qwak.fitness.service.GpuType\x12\x12\n\ngpu_amount\x18\x02 \x01(\x05\x12\x0b\n\x03\x63pu\x18\x03 \x01(\x02\x12\x15\n\rmemory_amount\x18\x04 \x01(\x05\x12\x36\n\x0cmemory_units\x18\x05 \x01(\x0e\x32 .qwak.fitness.service.MemoryUnit*7\n\nMemoryUnit\x12\x17\n\x13UNKNOWN_MEMORY_UNIT\x10\x00\x12\x07\n\x03MIB\x10\x01\x12\x07\n\x03GIB\x10\x02*\x82\x03\n\x07GpuType\x12\x0b\n\x07INVALID\x10\x00\x12\x0e\n\nNVIDIA_K80\x10\x01\x12\x0f\n\x0bNVIDIA_V100\x10\x02\x12\x0f\n\x0bNVIDIA_A100\x10\x03\x12\r\n\tNVIDIA_T4\x10\x04\x12\x0f\n\x0bNVIDIA_A10G\x10\x05\x12\r\n\tNVIDIA_L4\x10\x06\x12\x14\n\x10NVIDIA_T4_1_4_15\x10\x07\x12\x14\n\x10NVIDIA_T4_1_8_30\x10\x08\x12\x15\n\x11NVIDIA_T4_1_16_60\x10\t\x12\x1e\n\x1aNVIDIA_A100_80GB_8_96_1360\x10\n\x12\x16\n\x12NVIDIA_V100_1_8_52\x10\x0b\x12\x18\n\x14NVIDIA_V100_4_32_208\x10\x0c\x12\x13\n\x0fNVIDIA_H100_NVL\x10\r\x12\x0e\n\nNVIDIA_A10\x10\x0e\x12\x13\n\x0fNVIDIA_H100_SXM\x10\x0f\x12\x14\n\x10NVIDIA_H100_40GB\x10\x10\x12\x14\n\x10NVIDIA_H100_80GB\x10\x11\x12\x0e\n\nNVIDIA_M60\x10\x12\x42#\n\x1f\x63om.qwak.ai.fitness.service.apiP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.fitness_service.constructs_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\037com.qwak.ai.fitness.service.apiP\001'
  _globals['_MEMORYUNIT']._serialized_start=1043
  _globals['_MEMORYUNIT']._serialized_end=1098
  _globals['_GPUTYPE']._serialized_start=1101
  _globals['_GPUTYPE']._serialized_end=1487
  _globals['_BUILDPROPERTIES']._serialized_start=64
  _globals['_BUILDPROPERTIES']._serialized_end=226
  _globals['_MODELURISPEC']._serialized_start=229
  _globals['_MODELURISPEC']._serialized_end=370
  _globals['_BUILDENV']._serialized_start=372
  _globals['_BUILDENV']._serialized_end=488
  _globals['_DOCKERENV']._serialized_start=491
  _globals['_DOCKERENV']._serialized_end=629
  _globals['_PYTHONENV']._serialized_start=631
  _globals['_PYTHONENV']._serialized_end=755
  _globals['_CPURESOURCES']._serialized_start=757
  _globals['_CPURESOURCES']._serialized_end=863
  _globals['_GPURESOURCES']._serialized_start=866
  _globals['_GPURESOURCES']._serialized_end=1041
# @@protoc_insertion_point(module_scope)



import time


def create_qwak_timer(name):
    try:
        start_time = time.time()
        yield
    finally:
        elapsed_secs = time.time() - start_time
        print(f"elapsed time: {elapsed_secs} seconds")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/feature_store/serving/management.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.feature_store.serving import metadata_pb2 as qwak_dot_feature__store_dot_serving_dot_metadata__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+qwak/feature_store/serving/management.proto\x12%com.qwak.ai.feature.store.serving.api\x1a)qwak/feature_store/serving/metadata.proto\"<\n#GetFeaturesetMemoryUsageByIdRequest\x12\x15\n\rfeatureset_id\x18\x01 \x01(\t\"}\n$GetFeaturesetMemoryUsageByIdResponse\x12U\n\x05usage\x18\x01 \x01(\x0b\x32\x46.com.qwak.ai.feature.store.serving.metadata.FeaturesetMemoryUsageStats\".\n,GetFeaturesetMemoryUsageByEnvironmentRequest\"\x8c\x01\n-GetFeaturesetMemoryUsageByEnvironmentResponse\x12[\n\x0busage_stats\x18\x01 \x03(\x0b\x32\x46.com.qwak.ai.feature.store.serving.metadata.FeaturesetMemoryUsageStats2\xac\x03\n\x1b\x46\x65\x61turesetManagementService\x12\xb7\x01\n\x1cGetFeaturesetMemoryUsageById\x12J.com.qwak.ai.feature.store.serving.api.GetFeaturesetMemoryUsageByIdRequest\x1aK.com.qwak.ai.feature.store.serving.api.GetFeaturesetMemoryUsageByIdResponse\x12\xd2\x01\n%GetFeaturesetMemoryUsageByEnvironment\x12S.com.qwak.ai.feature.store.serving.api.GetFeaturesetMemoryUsageByEnvironmentRequest\x1aT.com.qwak.ai.feature.store.serving.api.GetFeaturesetMemoryUsageByEnvironmentResponseB\'\n%com.qwak.ai.feature.store.serving.apib\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.feature_store.serving.management_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n%com.qwak.ai.feature.store.serving.api'
  _globals['_GETFEATURESETMEMORYUSAGEBYIDREQUEST']._serialized_start=129
  _globals['_GETFEATURESETMEMORYUSAGEBYIDREQUEST']._serialized_end=189
  _globals['_GETFEATURESETMEMORYUSAGEBYIDRESPONSE']._serialized_start=191
  _globals['_GETFEATURESETMEMORYUSAGEBYIDRESPONSE']._serialized_end=316
  _globals['_GETFEATURESETMEMORYUSAGEBYENVIRONMENTREQUEST']._serialized_start=318
  _globals['_GETFEATURESETMEMORYUSAGEBYENVIRONMENTREQUEST']._serialized_end=364
  _globals['_GETFEATURESETMEMORYUSAGEBYENVIRONMENTRESPONSE']._serialized_start=367
  _globals['_GETFEATURESETMEMORYUSAGEBYENVIRONMENTRESPONSE']._serialized_end=507
  _globals['_FEATURESETMANAGEMENTSERVICE']._serialized_start=510
  _globals['_FEATURESETMANAGEMENTSERVICE']._serialized_end=938
# @@protoc_insertion_point(module_scope)



import os
from pathlib import Path
from typing import Optional

from qwak.inner.const import QwakConstants
from qwak.inner.di_configuration import QwakContainer
from qwak.inner.tool.grpc.grpc_tools import validate_grpc_address
from qwak.tools.logger import get_qwak_logger


logger = get_qwak_logger()

__DEFAULT_CONFIG_FILE_PATH: Path = Path(__file__).parent / "config.yml"


def wire_dependencies():
    container = QwakContainer()

    container.config.from_yaml(__DEFAULT_CONFIG_FILE_PATH)
    control_plane_grpc_address_override: Optional[str] = os.getenv(
        QwakConstants.CONTROL_PLANE_GRPC_ADDRESS_ENVAR_NAME
    )

    if control_plane_grpc_address_override:
        validate_grpc_address(control_plane_grpc_address_override)
        __override_control_plane_grpc_address(
            container, control_plane_grpc_address_override
        )

    from qwak.clients import (
        administration,
        alert_management,
        alerts_registry,
        analytics,
        audience,
        automation_management,
        autoscaling,
        batch_job_management,
        build_orchestrator,
        data_versioning,
        deployment,
        feature_store,
        file_versioning,
        instance_template,
        integration_management,
        kube_deployment_captain,
        logging_client,
        model_management,
        project,
        prompt_manager,
        system_secret,
        user_application_instance,
        workspace_manager,
    )

    container.wire(
        packages=[
            administration,
            alert_management,
            audience,
            automation_management,
            autoscaling,
            analytics,
            batch_job_management,
            build_orchestrator,
            data_versioning,
            deployment,
            file_versioning,
            instance_template,
            kube_deployment_captain,
            logging_client,
            model_management,
            project,
            feature_store,
            user_application_instance,
            alerts_registry,
            workspace_manager,
            integration_management,
            system_secret,
            prompt_manager,
        ]
    )

    return container


def __override_control_plane_grpc_address(
    container: "QwakContainer", control_plane_grpc_address_override: str
):
    logger.debug(
        "Overriding control plane gRPC address from environment variable to %s.",
        control_plane_grpc_address_override,
    )
    container.config.grpc.core.address.from_value(
        control_plane_grpc_address_override.strip()
    )



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import qwak.automation.v1.automation_execution_pb2
import qwak.automation.v1.automation_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class CreateAutomationRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AUTOMATION_SPEC_FIELD_NUMBER: builtins.int
    @property
    def automation_spec(self) -> qwak.automation.v1.automation_pb2.AutomationSpec: ...
    def __init__(
        self,
        *,
        automation_spec: qwak.automation.v1.automation_pb2.AutomationSpec | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["automation_spec", b"automation_spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["automation_spec", b"automation_spec"]) -> None: ...

global___CreateAutomationRequest = CreateAutomationRequest

class CreateAutomationResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AUTOMATION_ID_FIELD_NUMBER: builtins.int
    automation_id: builtins.str
    def __init__(
        self,
        *,
        automation_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["automation_id", b"automation_id"]) -> None: ...

global___CreateAutomationResponse = CreateAutomationResponse

class UpdateAutomationRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AUTOMATION_ID_FIELD_NUMBER: builtins.int
    AUTOMATION_SPEC_FIELD_NUMBER: builtins.int
    automation_id: builtins.str
    @property
    def automation_spec(self) -> qwak.automation.v1.automation_pb2.AutomationSpec: ...
    def __init__(
        self,
        *,
        automation_id: builtins.str = ...,
        automation_spec: qwak.automation.v1.automation_pb2.AutomationSpec | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["automation_spec", b"automation_spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["automation_id", b"automation_id", "automation_spec", b"automation_spec"]) -> None: ...

global___UpdateAutomationRequest = UpdateAutomationRequest

class UpdateAutomationResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___UpdateAutomationResponse = UpdateAutomationResponse

class DeleteAutomationRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AUTOMATION_ID_FIELD_NUMBER: builtins.int
    automation_id: builtins.str
    """The automation id to delete."""
    def __init__(
        self,
        *,
        automation_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["automation_id", b"automation_id"]) -> None: ...

global___DeleteAutomationRequest = DeleteAutomationRequest

class DeleteAutomationResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___DeleteAutomationResponse = DeleteAutomationResponse

class ListAutomationsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODEL_ID_FIELD_NUMBER: builtins.int
    model_id: builtins.str
    def __init__(
        self,
        *,
        model_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["model_id", b"model_id"]) -> None: ...

global___ListAutomationsRequest = ListAutomationsRequest

class ListAutomationsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AUTOMATIONS_FIELD_NUMBER: builtins.int
    @property
    def automations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[qwak.automation.v1.automation_pb2.Automation]: ...
    def __init__(
        self,
        *,
        automations: collections.abc.Iterable[qwak.automation.v1.automation_pb2.Automation] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["automations", b"automations"]) -> None: ...

global___ListAutomationsResponse = ListAutomationsResponse

class GetAutomationRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AUTOMATION_ID_FIELD_NUMBER: builtins.int
    automation_id: builtins.str
    """The automation id to get."""
    def __init__(
        self,
        *,
        automation_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["automation_id", b"automation_id"]) -> None: ...

global___GetAutomationRequest = GetAutomationRequest

class GetAutomationResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AUTOMATION_FIELD_NUMBER: builtins.int
    @property
    def automation(self) -> qwak.automation.v1.automation_pb2.Automation: ...
    def __init__(
        self,
        *,
        automation: qwak.automation.v1.automation_pb2.Automation | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["automation", b"automation"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["automation", b"automation"]) -> None: ...

global___GetAutomationResponse = GetAutomationResponse

class GetAutomationByNameRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AUTOMATION_NAME_FIELD_NUMBER: builtins.int
    automation_name: builtins.str
    """The automation name to get."""
    def __init__(
        self,
        *,
        automation_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["automation_name", b"automation_name"]) -> None: ...

global___GetAutomationByNameRequest = GetAutomationByNameRequest

class GetAutomationByNameResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AUTOMATION_FIELD_NUMBER: builtins.int
    @property
    def automation(self) -> qwak.automation.v1.automation_pb2.Automation: ...
    def __init__(
        self,
        *,
        automation: qwak.automation.v1.automation_pb2.Automation | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["automation", b"automation"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["automation", b"automation"]) -> None: ...

global___GetAutomationByNameResponse = GetAutomationByNameResponse

class ListAutomationExecutionsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AUTOMATION_ID_FIELD_NUMBER: builtins.int
    DATE_RANGE_FIELD_NUMBER: builtins.int
    automation_id: builtins.str
    """The automation id to get executions for."""
    @property
    def date_range(self) -> qwak.automation.v1.automation_execution_pb2.DateRange:
        """Date range to fetch executions for"""
    def __init__(
        self,
        *,
        automation_id: builtins.str = ...,
        date_range: qwak.automation.v1.automation_execution_pb2.DateRange | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["date_range", b"date_range"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["automation_id", b"automation_id", "date_range", b"date_range"]) -> None: ...

global___ListAutomationExecutionsRequest = ListAutomationExecutionsRequest

class ListAutomationExecutionsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AUTOMATION_EXECUTIONS_FIELD_NUMBER: builtins.int
    @property
    def automation_executions(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[qwak.automation.v1.automation_execution_pb2.AutomationExecutionMessage]:
        """The automation executions."""
    def __init__(
        self,
        *,
        automation_executions: collections.abc.Iterable[qwak.automation.v1.automation_execution_pb2.AutomationExecutionMessage] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["automation_executions", b"automation_executions"]) -> None: ...

global___ListAutomationExecutionsResponse = ListAutomationExecutionsResponse

class ListLastAutomationExecutionsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODEL_ID_FIELD_NUMBER: builtins.int
    model_id: builtins.str
    """The model id to get executions for."""
    def __init__(
        self,
        *,
        model_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["model_id", b"model_id"]) -> None: ...

global___ListLastAutomationExecutionsRequest = ListLastAutomationExecutionsRequest

class ListLastAutomationExecutionsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class LastAutomationExecutionByAutomationIdEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> qwak.automation.v1.automation_execution_pb2.AutomationExecutionMessage: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: qwak.automation.v1.automation_execution_pb2.AutomationExecutionMessage | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    LAST_AUTOMATION_EXECUTION_BY_AUTOMATION_ID_FIELD_NUMBER: builtins.int
    @property
    def last_automation_execution_by_automation_id(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, qwak.automation.v1.automation_execution_pb2.AutomationExecutionMessage]:
        """Automation ids to their last automation executions."""
    def __init__(
        self,
        *,
        last_automation_execution_by_automation_id: collections.abc.Mapping[builtins.str, qwak.automation.v1.automation_execution_pb2.AutomationExecutionMessage] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["last_automation_execution_by_automation_id", b"last_automation_execution_by_automation_id"]) -> None: ...

global___ListLastAutomationExecutionsResponse = ListLastAutomationExecutionsResponse

class RegisterAutomationExecutionRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AUTOMATION_ID_FIELD_NUMBER: builtins.int
    EXECUTION_ID_FIELD_NUMBER: builtins.int
    automation_id: builtins.str
    """The automation id to register."""
    execution_id: builtins.str
    """The execution id."""
    def __init__(
        self,
        *,
        automation_id: builtins.str = ...,
        execution_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["automation_id", b"automation_id", "execution_id", b"execution_id"]) -> None: ...

global___RegisterAutomationExecutionRequest = RegisterAutomationExecutionRequest

class RegisterAutomationExecutionResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___RegisterAutomationExecutionResponse = RegisterAutomationExecutionResponse

class UpdateAutomationExecutionRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EXECUTION_ID_FIELD_NUMBER: builtins.int
    RUN_DETAILS_FIELD_NUMBER: builtins.int
    execution_id: builtins.str
    """The execution id to update."""
    @property
    def run_details(self) -> qwak.automation.v1.automation_execution_pb2.ExecutionRunDetailsMessage: ...
    def __init__(
        self,
        *,
        execution_id: builtins.str = ...,
        run_details: qwak.automation.v1.automation_execution_pb2.ExecutionRunDetailsMessage | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["run_details", b"run_details"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["execution_id", b"execution_id", "run_details", b"run_details"]) -> None: ...

global___UpdateAutomationExecutionRequest = UpdateAutomationExecutionRequest

class UpdateAutomationExecutionResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___UpdateAutomationExecutionResponse = UpdateAutomationExecutionResponse

class ToggleAutomationActivityRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AUTOMATION_ID_FIELD_NUMBER: builtins.int
    IS_ENABLED_FIELD_NUMBER: builtins.int
    automation_id: builtins.str
    """The automation id to update"""
    is_enabled: builtins.bool
    """If the automation is enabled."""
    def __init__(
        self,
        *,
        automation_id: builtins.str = ...,
        is_enabled: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["automation_id", b"automation_id", "is_enabled", b"is_enabled"]) -> None: ...

global___ToggleAutomationActivityRequest = ToggleAutomationActivityRequest

class ToggleAutomationActivityResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ToggleAutomationActivityResponse = ToggleAutomationActivityResponse

class RunAutomationRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AUTOMATION_ID_FIELD_NUMBER: builtins.int
    automation_id: builtins.str
    """The automation id to run now"""
    def __init__(
        self,
        *,
        automation_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["automation_id", b"automation_id"]) -> None: ...

global___RunAutomationRequest = RunAutomationRequest

class RunAutomationResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___RunAutomationResponse = RunAutomationResponse



from dataclasses import dataclass
from typing import Optional
import warnings

from _qwak_proto.qwak.feature_store.sources.batch_pb2 import (
    BatchSource as ProtoBatchSource,
    SnowflakeSource as ProtoSnowflakeSource,
)
from _qwak_proto.qwak.feature_store.sources.data_source_pb2 import (
    DataSourceSpec as ProtoDataSourceSpec,
)
from qwak.exceptions import QwakException
from qwak.feature_store.data_sources.batch._batch import BaseBatchSource

warnings.simplefilter("once", DeprecationWarning)


@dataclass
class SnowflakeSource(BaseBatchSource):
    host: str
    username_secret_name: str
    database: str
    schema: str
    warehouse: str
    password_secret_name: Optional[str] = None
    pem_private_key_secret_name: Optional[str] = None
    table: Optional[str] = None
    query: Optional[str] = None
    repository: Optional[str] = None

    def __post_init__(self):
        self._validate()

    def _validate(self):
        if not self.username_secret_name:
            raise QwakException("username_secret_name must be set!")

        if not self.database:
            raise QwakException("database must be set!")

        if not self.schema:
            raise QwakException("schema must be set!")

        if self.password_secret_name:
            warnings.warn(
                "Snowflake basic authentication is deprecated and should not be used. Use key-pair authentication instead.",
                DeprecationWarning,
            )

        no_unique_source_exception_message = "Only one of query or table may be set"
        has_table = bool(self.table)
        has_query = bool(self.query)
        no_source_set = not (has_table or has_query)
        both_source_set = has_table and has_query
        if no_source_set or both_source_set:
            raise QwakException(no_unique_source_exception_message)

        no_unique_auth_exception_message = "Exactly one of 'password_secret_name' or 'pem_private_key_secret_name' must be set"
        has_password_secret = bool(self.password_secret_name)
        has_pem_private_key_secret = bool(self.pem_private_key_secret_name)
        no_auth_set = not (has_password_secret or has_pem_private_key_secret)
        both_auth_set = has_password_secret and has_pem_private_key_secret
        if no_auth_set or both_auth_set:
            raise QwakException(no_unique_auth_exception_message)

    def _to_proto(self, artifact_url: Optional[str] = None):
        return ProtoDataSourceSpec(
            data_source_repository_name=self.repository,
            batch_source=ProtoBatchSource(
                name=self.name,
                description=self.description,
                date_created_column=self.date_created_column,
                snowflakeSource=ProtoSnowflakeSource(
                    host=self.host,
                    username_secret_name=self.username_secret_name,
                    password_secret_name=self.password_secret_name,
                    pem_private_key_secret_name=self.pem_private_key_secret_name,
                    database=self.database,
                    schema=self.schema,
                    warehouse=self.warehouse,
                    table=self.table,
                    query=self.query,
                ),
            ),
        )

    @classmethod
    def _from_proto(cls, proto):
        snowflake = proto.snowflakeSource
        return cls(
            name=proto.name,
            date_created_column=proto.date_created_column,
            description=proto.description,
            host=snowflake.host,
            username_secret_name=snowflake.username_secret_name,
            password_secret_name=snowflake.password_secret_name,
            pem_private_key_secret_name=snowflake.pem_private_key_secret_name,
            database=snowflake.database,
            schema=snowflake.schema,
            warehouse=snowflake.warehouse,
            table=snowflake.table,
            query=snowflake.query,
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/offline/serving/v1/offline_serving_async_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from _qwak_proto.qwak.offline.serving.v1 import population_pb2 as qwak_dot_offline_dot_serving_dot_v1_dot_population__pb2
from _qwak_proto.qwak.offline.serving.v1 import feature_values_pb2 as qwak_dot_offline_dot_serving_dot_v1_dot_feature__values__pb2
from _qwak_proto.qwak.offline.serving.v1 import options_pb2 as qwak_dot_offline_dot_serving_dot_v1_dot_options__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n;qwak/offline/serving/v1/offline_serving_async_service.proto\x12%qwak.feature.store.offline.serving.v1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a(qwak/offline/serving/v1/population.proto\x1a,qwak/offline/serving/v1/feature_values.proto\x1a%qwak/offline/serving/v1/options.proto\"{\n\x17GetFileUploadUrlRequest\x12X\n\npopulation\x18\x01 \x01(\x0b\x32\x42.qwak.feature.store.offline.serving.v1.PopulationFileUploadUrlTypeH\x00\x42\x06\n\x04type\"3\n\x18GetFileUploadUrlResponse\x12\x17\n\x0f\x66ile_upload_url\x18\x01 \x01(\t\"\xd5\x02\n\x17GetFeatureValuesRequest\x12K\n\x08\x66\x65\x61tures\x18\x01 \x03(\x0b\x32\x39.qwak.feature.store.offline.serving.v1.FeaturesetFeatures\x12J\n\npopulation\x18\x02 \x01(\x0b\x32\x36.qwak.feature.store.offline.serving.v1.TimedPopulation\x12M\n\x12result_file_format\x18\x03 \x01(\x0e\x32\x31.qwak.feature.store.offline.serving.v1.FileFormat\x12R\n\x07options\x18\x04 \x01(\x0b\x32\x41.qwak.feature.store.offline.serving.v1.OfflineServingQueryOptions\".\n\x18GetFeatureValuesResponse\x12\x12\n\nrequest_id\x18\x01 \x01(\t\"\xdc\x03\n\x1eGetFeatureValuesInRangeRequest\x12K\n\x08\x66\x65\x61tures\x18\x01 \x01(\x0b\x32\x39.qwak.feature.store.offline.serving.v1.FeaturesetFeatures\x12\x34\n\x10lower_time_bound\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x34\n\x10upper_time_bound\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12G\n\npopulation\x18\x04 \x01(\x0b\x32\x31.qwak.feature.store.offline.serving.v1.PopulationH\x00\x12M\n\x12result_file_format\x18\x05 \x01(\x0e\x32\x31.qwak.feature.store.offline.serving.v1.FileFormat\x12R\n\x07options\x18\x06 \x01(\x0b\x32\x41.qwak.feature.store.offline.serving.v1.OfflineServingQueryOptionsB\x15\n\x13optional_population\"5\n\x1fGetFeatureValuesInRangeResponse\x12\x12\n\nrequest_id\x18\x01 \x01(\t\"3\n\x1dGetFeatureValuesResultRequest\x12\x12\n\nrequest_id\x18\x01 \x01(\t\"\x9b\x02\n\x1eGetFeatureValuesResultResponse\x12Q\n\x06status\x18\x01 \x01(\x0e\x32\x41.qwak.feature.store.offline.serving.v1.FeatureValuesRequestStatus\x12K\n\rlink_to_files\x18\x02 \x01(\x0b\x32\x32.qwak.feature.store.offline.serving.v1.LinkToFilesH\x00\x12N\n\x0e\x66\x61ilure_reason\x18\x03 \x01(\x0b\x32\x34.qwak.feature.store.offline.serving.v1.FailureReasonH\x00\x42\t\n\x07payload\" \n\rFailureReason\x12\x0f\n\x07message\x18\x01 \x01(\t\"$\n\x0bLinkToFiles\x12\x15\n\rlink_to_files\x18\x01 \x03(\t*\xf6\x01\n\x1a\x46\x65\x61tureValuesRequestStatus\x12)\n%FEATURE_VALUES_REQUEST_STATUS_INVALID\x10\x00\x12+\n\'FEATURE_VALUES_REQUEST_STATUS_SUCCEEDED\x10\x01\x12)\n%FEATURE_VALUES_REQUEST_STATUS_PENDING\x10\x02\x12+\n\'FEATURE_VALUES_REQUEST_STATUS_CANCELLED\x10\x03\x12(\n$FEATURE_VALUES_REQUEST_STATUS_FAILED\x10\x04*:\n\nFileFormat\x12\x17\n\x13\x46ILE_FORMAT_INVALID\x10\x00\x12\x13\n\x0f\x46ILE_FORMAT_CSV\x10\x01\x32\xa7\x05\n&FeatureStoreOfflineServingAsyncService\x12\x93\x01\n\x10GetFileUploadUrl\x12>.qwak.feature.store.offline.serving.v1.GetFileUploadUrlRequest\x1a?.qwak.feature.store.offline.serving.v1.GetFileUploadUrlResponse\x12\x93\x01\n\x10GetFeatureValues\x12>.qwak.feature.store.offline.serving.v1.GetFeatureValuesRequest\x1a?.qwak.feature.store.offline.serving.v1.GetFeatureValuesResponse\x12\xa8\x01\n\x17GetFeatureValuesInRange\x12\x45.qwak.feature.store.offline.serving.v1.GetFeatureValuesInRangeRequest\x1a\x46.qwak.feature.store.offline.serving.v1.GetFeatureValuesInRangeResponse\x12\xa5\x01\n\x16GetFeatureValuesResult\x12\x44.qwak.feature.store.offline.serving.v1.GetFeatureValuesResultRequest\x1a\x45.qwak.feature.store.offline.serving.v1.GetFeatureValuesResultResponseB&\n\"com.qwak.ai.offline.serving.api.v1P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.offline.serving.v1.offline_serving_async_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\"com.qwak.ai.offline.serving.api.v1P\001'
  _globals['_FEATUREVALUESREQUESTSTATUS']._serialized_start=1778
  _globals['_FEATUREVALUESREQUESTSTATUS']._serialized_end=2024
  _globals['_FILEFORMAT']._serialized_start=2026
  _globals['_FILEFORMAT']._serialized_end=2084
  _globals['_GETFILEUPLOADURLREQUEST']._serialized_start=262
  _globals['_GETFILEUPLOADURLREQUEST']._serialized_end=385
  _globals['_GETFILEUPLOADURLRESPONSE']._serialized_start=387
  _globals['_GETFILEUPLOADURLRESPONSE']._serialized_end=438
  _globals['_GETFEATUREVALUESREQUEST']._serialized_start=441
  _globals['_GETFEATUREVALUESREQUEST']._serialized_end=782
  _globals['_GETFEATUREVALUESRESPONSE']._serialized_start=784
  _globals['_GETFEATUREVALUESRESPONSE']._serialized_end=830
  _globals['_GETFEATUREVALUESINRANGEREQUEST']._serialized_start=833
  _globals['_GETFEATUREVALUESINRANGEREQUEST']._serialized_end=1309
  _globals['_GETFEATUREVALUESINRANGERESPONSE']._serialized_start=1311
  _globals['_GETFEATUREVALUESINRANGERESPONSE']._serialized_end=1364
  _globals['_GETFEATUREVALUESRESULTREQUEST']._serialized_start=1366
  _globals['_GETFEATUREVALUESRESULTREQUEST']._serialized_end=1417
  _globals['_GETFEATUREVALUESRESULTRESPONSE']._serialized_start=1420
  _globals['_GETFEATUREVALUESRESULTRESPONSE']._serialized_end=1703
  _globals['_FAILUREREASON']._serialized_start=1705
  _globals['_FAILUREREASON']._serialized_end=1737
  _globals['_LINKTOFILES']._serialized_start=1739
  _globals['_LINKTOFILES']._serialized_end=1775
  _globals['_FEATURESTOREOFFLINESERVINGASYNCSERVICE']._serialized_start=2087
  _globals['_FEATURESTOREOFFLINESERVINGASYNCSERVICE']._serialized_end=2766
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/self_service/user/v1/api_key.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'qwak/self_service/user/v1/api_key.proto\x12\x19qwak.self_service.user.v1\"\x1f\n\rApiKeyDetails\x12\x0e\n\x06\x65xists\x18\x01 \x01(\x08\x42\xaf\x01\n\x1f\x63om.qwak.ai.self_service.v1.apiP\x01Z\x89\x01github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/self_service/user/v1;self_service_user_v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.self_service.user.v1.api_key_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\037com.qwak.ai.self_service.v1.apiP\001Z\211\001github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/self_service/user/v1;self_service_user_v1'
  _globals['_APIKEYDETAILS']._serialized_start=70
  _globals['_APIKEYDETAILS']._serialized_end=101
# @@protoc_insertion_point(module_scope)



from typing import Dict

from _qwak_proto.qwak.self_service.user.v1.user_service_pb2 import (
    GenerateApiKeyResponse,
    RevokeApiKeyResponse,
)
from _qwak_proto.qwak.self_service.user.v1.user_service_pb2_grpc import (
    UserServiceServicer,
)

API_KEY_FORMAT = "apikey-for-{}-{}"


class SelfServiceUserServiceMock(UserServiceServicer):
    def __init__(self):
        super(SelfServiceUserServiceMock, self).__init__()
        self.apikeys: Dict[(str, str), str] = dict()

    def RevokeApiKey(self, request, context) -> RevokeApiKeyResponse:
        if self.apikeys.get((request.user_id, request.environment_id)):
            self.apikeys.pop((request.user_id, request.environment_id))
            return RevokeApiKeyResponse(status=RevokeApiKeyResponse.Status.REVOKED)
        else:
            return RevokeApiKeyResponse(
                status=RevokeApiKeyResponse.Status.INVALID_USER_STATUS
            )

    def GenerateApiKey(self, request, context) -> GenerateApiKeyResponse:
        api_key = API_KEY_FORMAT.format(request.user_id, request.environment_id)
        self.apikeys[(request.user_id, request.environment_id)] = api_key
        return GenerateApiKeyResponse(api_key=api_key)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/self_service/account/v0/account_membership_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.self_service.account.v0 import account_membership_pb2 as qwak_dot_self__service_dot_account_dot_v0_dot_account__membership__pb2
from _qwak_proto.qwak.self_service.user.v1 import user_pb2 as qwak_dot_self__service_dot_user_dot_v1_dot_user__pb2
from _qwak_proto.qwak.administration.v0.users import user_pb2 as qwak_dot_administration_dot_v0_dot_users_dot_user__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n=qwak/self_service/account/v0/account_membership_service.proto\x12\x1cqwak.self_service.account.v0\x1a\x35qwak/self_service/account/v0/account_membership.proto\x1a$qwak/self_service/user/v1/user.proto\x1a\'qwak/administration/v0/users/user.proto\" \n\x1eListAccountUserProfilesRequest\"`\n\x1fListAccountUserProfilesResponse\x12=\n\ruser_profiles\x18\x01 \x03(\x0b\x32&.qwak.self_service.user.v1.UserProfile\"d\n\x15UpdateUserRoleRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12?\n\x0c\x61\x63\x63ount_role\x18\x02 \x01(\x0e\x32).qwak.administration.user.QwakAccountRole\"V\n\x16UpdateUserRoleResponse\x12<\n\x0cuser_profile\x18\x01 \x01(\x0b\x32&.qwak.self_service.user.v1.UserProfile\"L\n%UpdateUserAssignedEnvironmentsRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x17\n\x0f\x65nvironment_ids\x18\x02 \x03(\t\"f\n&UpdateUserAssignedEnvironmentsResponse\x12<\n\x0cuser_profile\x18\x01 \x01(\x0b\x32&.qwak.self_service.user.v1.UserProfile\"*\n\x1cRemoveUserFromAccountRequest\x12\n\n\x02id\x18\x01 \x01(\t\"\x1f\n\x1dRemoveUserFromAccountResponse\"Z\n\x1aInviteToJoinAccountRequest\x12<\n\x07options\x18\x01 \x03(\x0b\x32+.qwak.self_service.account.v0.InviteOptions\"n\n\x1bInviteToJoinAccountResponse\x12O\n\x0binvitations\x18\x01 \x03(\x0b\x32:.qwak.self_service.account.v0.JoiningInvitationDescription\"\x8a\x01\n\x17UpdateInvitationRequest\x12\x15\n\rinvitation_id\x18\x01 \x01(\t\x12?\n\x0c\x61\x63\x63ount_role\x18\x02 \x01(\x0e\x32).qwak.administration.user.QwakAccountRole\x12\x17\n\x0f\x65nvironment_ids\x18\x03 \x03(\t\"j\n\x18UpdateInvitationResponse\x12N\n\ninvitation\x18\x01 \x01(\x0b\x32:.qwak.self_service.account.v0.JoiningInvitationDescription\"0\n\x17\x41\x63\x63\x65ptInvitationRequest\x12\x15\n\rinvitation_id\x18\x01 \x01(\t\"\x1a\n\x18\x41\x63\x63\x65ptInvitationResponse\"7\n\x1e\x43\x61ncelJoiningInvitationRequest\x12\x15\n\rinvitation_id\x18\x01 \x01(\t\"!\n\x1f\x43\x61ncelJoiningInvitationResponse\"\x1f\n\x1dListAccountInvitationsRequest\"q\n\x1eListAccountInvitationsResponse\x12O\n\x0binvitations\x18\x01 \x03(\x0b\x32:.qwak.self_service.account.v0.JoiningInvitationDescription\"\x1c\n\x1aListUserInvitationsRequest\"n\n\x1bListUserInvitationsResponse\x12O\n\x0binvitations\x18\x01 \x03(\x0b\x32:.qwak.self_service.account.v0.JoiningInvitationDescription\"\x1c\n\x1aListUserMembershipsRequest\"\xbb\x01\n\x1bListUserMembershipsResponse\x12K\n\ruser_accounts\x18\x01 \x03(\x0b\x32\x34.qwak.self_service.account.v0.UserAccountDescription\x12O\n\x0binvitations\x18\x02 \x03(\x0b\x32:.qwak.self_service.account.v0.JoiningInvitationDescription2\xcf\x0c\n\x18\x41\x63\x63ountMembershipService\x12\x96\x01\n\x17ListAccountUserProfiles\x12<.qwak.self_service.account.v0.ListAccountUserProfilesRequest\x1a=.qwak.self_service.account.v0.ListAccountUserProfilesResponse\x12{\n\x0eUpdateUserRole\x12\x33.qwak.self_service.account.v0.UpdateUserRoleRequest\x1a\x34.qwak.self_service.account.v0.UpdateUserRoleResponse\x12\xab\x01\n\x1eUpdateUserAssignedEnvironments\x12\x43.qwak.self_service.account.v0.UpdateUserAssignedEnvironmentsRequest\x1a\x44.qwak.self_service.account.v0.UpdateUserAssignedEnvironmentsResponse\x12\x90\x01\n\x15RemoveUserFromAccount\x12:.qwak.self_service.account.v0.RemoveUserFromAccountRequest\x1a;.qwak.self_service.account.v0.RemoveUserFromAccountResponse\x12\x8a\x01\n\x13InviteToJoinAccount\x12\x38.qwak.self_service.account.v0.InviteToJoinAccountRequest\x1a\x39.qwak.self_service.account.v0.InviteToJoinAccountResponse\x12\x81\x01\n\x10UpdateInvitation\x12\x35.qwak.self_service.account.v0.UpdateInvitationRequest\x1a\x36.qwak.self_service.account.v0.UpdateInvitationResponse\x12\x81\x01\n\x10\x41\x63\x63\x65ptInvitation\x12\x35.qwak.self_service.account.v0.AcceptInvitationRequest\x1a\x36.qwak.self_service.account.v0.AcceptInvitationResponse\x12\x96\x01\n\x17\x43\x61ncelJoiningInvitation\x12<.qwak.self_service.account.v0.CancelJoiningInvitationRequest\x1a=.qwak.self_service.account.v0.CancelJoiningInvitationResponse\x12\x93\x01\n\x16ListAccountInvitations\x12;.qwak.self_service.account.v0.ListAccountInvitationsRequest\x1a<.qwak.self_service.account.v0.ListAccountInvitationsResponse\x12\x8a\x01\n\x13ListUserInvitations\x12\x38.qwak.self_service.account.v0.ListUserInvitationsRequest\x1a\x39.qwak.self_service.account.v0.ListUserInvitationsResponse\x12\x8a\x01\n\x13ListUserMemberships\x12\x38.qwak.self_service.account.v0.ListUserMembershipsRequest\x1a\x39.qwak.self_service.account.v0.ListUserMembershipsResponseB\xb9\x01\n#com.qwak.ai.self_service.account.v0P\x01Z\x8f\x01github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/self_service/account/v0;self_service_account_v0b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.self_service.account.v0.account_membership_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n#com.qwak.ai.self_service.account.v0P\001Z\217\001github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/self_service/account/v0;self_service_account_v0'
  _globals['_LISTACCOUNTUSERPROFILESREQUEST']._serialized_start=229
  _globals['_LISTACCOUNTUSERPROFILESREQUEST']._serialized_end=261
  _globals['_LISTACCOUNTUSERPROFILESRESPONSE']._serialized_start=263
  _globals['_LISTACCOUNTUSERPROFILESRESPONSE']._serialized_end=359
  _globals['_UPDATEUSERROLEREQUEST']._serialized_start=361
  _globals['_UPDATEUSERROLEREQUEST']._serialized_end=461
  _globals['_UPDATEUSERROLERESPONSE']._serialized_start=463
  _globals['_UPDATEUSERROLERESPONSE']._serialized_end=549
  _globals['_UPDATEUSERASSIGNEDENVIRONMENTSREQUEST']._serialized_start=551
  _globals['_UPDATEUSERASSIGNEDENVIRONMENTSREQUEST']._serialized_end=627
  _globals['_UPDATEUSERASSIGNEDENVIRONMENTSRESPONSE']._serialized_start=629
  _globals['_UPDATEUSERASSIGNEDENVIRONMENTSRESPONSE']._serialized_end=731
  _globals['_REMOVEUSERFROMACCOUNTREQUEST']._serialized_start=733
  _globals['_REMOVEUSERFROMACCOUNTREQUEST']._serialized_end=775
  _globals['_REMOVEUSERFROMACCOUNTRESPONSE']._serialized_start=777
  _globals['_REMOVEUSERFROMACCOUNTRESPONSE']._serialized_end=808
  _globals['_INVITETOJOINACCOUNTREQUEST']._serialized_start=810
  _globals['_INVITETOJOINACCOUNTREQUEST']._serialized_end=900
  _globals['_INVITETOJOINACCOUNTRESPONSE']._serialized_start=902
  _globals['_INVITETOJOINACCOUNTRESPONSE']._serialized_end=1012
  _globals['_UPDATEINVITATIONREQUEST']._serialized_start=1015
  _globals['_UPDATEINVITATIONREQUEST']._serialized_end=1153
  _globals['_UPDATEINVITATIONRESPONSE']._serialized_start=1155
  _globals['_UPDATEINVITATIONRESPONSE']._serialized_end=1261
  _globals['_ACCEPTINVITATIONREQUEST']._serialized_start=1263
  _globals['_ACCEPTINVITATIONREQUEST']._serialized_end=1311
  _globals['_ACCEPTINVITATIONRESPONSE']._serialized_start=1313
  _globals['_ACCEPTINVITATIONRESPONSE']._serialized_end=1339
  _globals['_CANCELJOININGINVITATIONREQUEST']._serialized_start=1341
  _globals['_CANCELJOININGINVITATIONREQUEST']._serialized_end=1396
  _globals['_CANCELJOININGINVITATIONRESPONSE']._serialized_start=1398
  _globals['_CANCELJOININGINVITATIONRESPONSE']._serialized_end=1431
  _globals['_LISTACCOUNTINVITATIONSREQUEST']._serialized_start=1433
  _globals['_LISTACCOUNTINVITATIONSREQUEST']._serialized_end=1464
  _globals['_LISTACCOUNTINVITATIONSRESPONSE']._serialized_start=1466
  _globals['_LISTACCOUNTINVITATIONSRESPONSE']._serialized_end=1579
  _globals['_LISTUSERINVITATIONSREQUEST']._serialized_start=1581
  _globals['_LISTUSERINVITATIONSREQUEST']._serialized_end=1609
  _globals['_LISTUSERINVITATIONSRESPONSE']._serialized_start=1611
  _globals['_LISTUSERINVITATIONSRESPONSE']._serialized_end=1721
  _globals['_LISTUSERMEMBERSHIPSREQUEST']._serialized_start=1723
  _globals['_LISTUSERMEMBERSHIPSREQUEST']._serialized_end=1751
  _globals['_LISTUSERMEMBERSHIPSRESPONSE']._serialized_start=1754
  _globals['_LISTUSERMEMBERSHIPSRESPONSE']._serialized_end=1941
  _globals['_ACCOUNTMEMBERSHIPSERVICE']._serialized_start=1944
  _globals['_ACCOUNTMEMBERSHIPSERVICE']._serialized_end=3559
# @@protoc_insertion_point(module_scope)



import time


def retry(
    f,
    args=None,
    kwargs=None,
    exceptions=Exception,
    attempts=-1,
    delay=0,
):
    _attempts = attempts
    args = args if args else list()
    kwargs = kwargs if kwargs else dict()
    while _attempts:
        try:
            return f(*args, **kwargs)
        except exceptions:
            _attempts -= 1
            if not _attempts:
                raise

            time.sleep(delay)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/administration/runtime_configuration/v0/observability_config.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.administration.runtime_configuration.v0.external import prometheus_config_pb2 as qwak_dot_administration_dot_runtime__configuration_dot_v0_dot_external_dot_prometheus__config__pb2
from _qwak_proto.qwak.administration.runtime_configuration.v0.external import victoriametrics_config_pb2 as qwak_dot_administration_dot_runtime__configuration_dot_v0_dot_external_dot_victoriametrics__config__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nGqwak/administration/runtime_configuration/v0/observability_config.proto\x12\x41qwak.administration.runtime_configuration.v0.observability_config\x1aMqwak/administration/runtime_configuration/v0/external/prometheus_config.proto\x1aRqwak/administration/runtime_configuration/v0/external/victoriametrics_config.proto\"\x83\x02\n\x1aObservabilityConfiguration\x12n\n\nmonitoring\x18\x01 \x01(\x0b\x32Z.qwak.administration.runtime_configuration.v0.observability_config.MonitoringConfiguration\x12u\n\x08\x61lerting\x18\x02 \x01(\x0b\x32\x63.qwak.administration.runtime_configuration.v0.observability_config.AlertingIntegrationConfiguration\"\xa2\x02\n\x17MonitoringConfiguration\x12v\n\nprometheus\x18\x01 \x01(\x0b\x32`.qwak.administration.runtime_configuration.v0.external.prometheus_config.PrometheusConfigurationH\x00\x12\x86\x01\n\x10victoria_metrics\x18\x02 \x01(\x0b\x32j.qwak.administration.runtime_configuration.v0.external.victoriametrics_config.VictoriaMetricsConfigurationH\x00\x42\x06\n\x04type\"\xab\x02\n AlertingIntegrationConfiguration\x12v\n\nprometheus\x18\x01 \x01(\x0b\x32`.qwak.administration.runtime_configuration.v0.external.prometheus_config.PrometheusConfigurationH\x00\x12\x86\x01\n\x10victoria_metrics\x18\x02 \x01(\x0b\x32j.qwak.administration.runtime_configuration.v0.external.victoriametrics_config.VictoriaMetricsConfigurationH\x00\x42\x06\n\x04typeB\xe1\x01\n:com.jfrog.ml.runtime_configuration.v0.observability_configP\x01Z\xa0\x01github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/administration/runtime_configuration/v0;runtime_configuration_v0b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.administration.runtime_configuration.v0.observability_config_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n:com.jfrog.ml.runtime_configuration.v0.observability_configP\001Z\240\001github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/administration/runtime_configuration/v0;runtime_configuration_v0'
  _globals['_OBSERVABILITYCONFIGURATION']._serialized_start=306
  _globals['_OBSERVABILITYCONFIGURATION']._serialized_end=565
  _globals['_MONITORINGCONFIGURATION']._serialized_start=568
  _globals['_MONITORINGCONFIGURATION']._serialized_end=858
  _globals['_ALERTINGINTEGRATIONCONFIGURATION']._serialized_start=861
  _globals['_ALERTINGINTEGRATIONCONFIGURATION']._serialized_end=1160
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.timestamp_pb2
import qwak.feature_store.features.execution_pb2
import qwak.feature_store.features.feature_set_types_pb2
import sys
import typing

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class RowLevelStreamingAggregationIngestion(google.protobuf.message.Message):
    """Row Level Ingestion"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURESET_NAME_FIELD_NUMBER: builtins.int
    featureset_name: builtins.str
    def __init__(
        self,
        *,
        featureset_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["featureset_name", b"featureset_name"]) -> None: ...

global___RowLevelStreamingAggregationIngestion = RowLevelStreamingAggregationIngestion

class CompactionStreamingAggregationIngestion(google.protobuf.message.Message):
    """Offline Ingestion"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURESET_NAME_FIELD_NUMBER: builtins.int
    SCHEDULED_COMPACTION_FIELD_NUMBER: builtins.int
    MANUAL_COMPACTION_FIELD_NUMBER: builtins.int
    featureset_name: builtins.str
    @property
    def scheduled_compaction(self) -> global___ScheduledCompaction: ...
    @property
    def manual_compaction(self) -> global___ManualCompaction: ...
    def __init__(
        self,
        *,
        featureset_name: builtins.str = ...,
        scheduled_compaction: global___ScheduledCompaction | None = ...,
        manual_compaction: global___ManualCompaction | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["manual_compaction", b"manual_compaction", "scheduled_compaction", b"scheduled_compaction", "trigger_type", b"trigger_type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["featureset_name", b"featureset_name", "manual_compaction", b"manual_compaction", "scheduled_compaction", b"scheduled_compaction", "trigger_type", b"trigger_type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["trigger_type", b"trigger_type"]) -> typing_extensions.Literal["scheduled_compaction", "manual_compaction"] | None: ...

global___CompactionStreamingAggregationIngestion = CompactionStreamingAggregationIngestion

class ScheduledCompaction(google.protobuf.message.Message):
    """Scheduled Compaction ingestion (e.g., AirFlow)"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ScheduledCompaction = ScheduledCompaction

class ManualCompaction(google.protobuf.message.Message):
    """Manual Compaction ingestion (that is, someone manually running from the UI)"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ManualCompaction = ManualCompaction

class StreamingAggregationBackfillIngestion(google.protobuf.message.Message):
    """Streaming Aggregation Backfill"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURESET_NAME_FIELD_NUMBER: builtins.int
    EXECUTION_SPEC_FIELD_NUMBER: builtins.int
    START_TIMESTAMP_FIELD_NUMBER: builtins.int
    END_TIMESTAMP_FIELD_NUMBER: builtins.int
    TRANSFORMATION_FIELD_NUMBER: builtins.int
    DATA_SOURCE_SPECS_FIELD_NUMBER: builtins.int
    featureset_name: builtins.str
    @property
    def execution_spec(self) -> qwak.feature_store.features.execution_pb2.ExecutionSpec: ...
    @property
    def start_timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def end_timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def transformation(self) -> qwak.feature_store.features.feature_set_types_pb2.Transformation: ...
    @property
    def data_source_specs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___BackfillDataSource]: ...
    def __init__(
        self,
        *,
        featureset_name: builtins.str = ...,
        execution_spec: qwak.feature_store.features.execution_pb2.ExecutionSpec | None = ...,
        start_timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        end_timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        transformation: qwak.feature_store.features.feature_set_types_pb2.Transformation | None = ...,
        data_source_specs: collections.abc.Iterable[global___BackfillDataSource] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["end_timestamp", b"end_timestamp", "execution_spec", b"execution_spec", "start_timestamp", b"start_timestamp", "transformation", b"transformation"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["data_source_specs", b"data_source_specs", "end_timestamp", b"end_timestamp", "execution_spec", b"execution_spec", "featureset_name", b"featureset_name", "start_timestamp", b"start_timestamp", "transformation", b"transformation"]) -> None: ...

global___StreamingAggregationBackfillIngestion = StreamingAggregationBackfillIngestion

class BackfillDataSource(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_SOURCE_NAME_FIELD_NUMBER: builtins.int
    TIME_RANGE_FIELD_NUMBER: builtins.int
    data_source_name: builtins.str
    @property
    def time_range(self) -> global___TimeRange: ...
    def __init__(
        self,
        *,
        data_source_name: builtins.str = ...,
        time_range: global___TimeRange | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["time_range", b"time_range"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["data_source_name", b"data_source_name", "time_range", b"time_range"]) -> None: ...

global___BackfillDataSource = BackfillDataSource

class TimeRange(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    START_TIMESTAMP_FIELD_NUMBER: builtins.int
    END_TIMESTAMP_FIELD_NUMBER: builtins.int
    @property
    def start_timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def end_timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    def __init__(
        self,
        *,
        start_timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        end_timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["end_optional", b"end_optional", "end_timestamp", b"end_timestamp", "start_optional", b"start_optional", "start_timestamp", b"start_timestamp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["end_optional", b"end_optional", "end_timestamp", b"end_timestamp", "start_optional", b"start_optional", "start_timestamp", b"start_timestamp"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["end_optional", b"end_optional"]) -> typing_extensions.Literal["end_timestamp"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["start_optional", b"start_optional"]) -> typing_extensions.Literal["start_timestamp"] | None: ...

global___TimeRange = TimeRange



import http
from typing import Optional

import requests
from requests.auth import AuthBase, HTTPBasicAuth

from frogml_storage.artifactory import ArtifactoryApi
from frogml_storage.logging import logger
from frogml_storage.authentication.models import BearerAuth, EmptyAuth
from frogml_storage.authentication.utils import (
    get_encrypted_password,
    save_auth_config,
)
from frogml_storage.authentication.models import AuthConfig
from frogml_storage.authentication.utils import (
    is_username_password_login,
    is_access_token_login,
    is_anonymous_login,
)


def run(auth_config: AuthConfig, anonymous: Optional[bool] = False) -> bool:
    if is_username_password_login(auth_config, anonymous):
        connection_validation_result = __login_by_username_password(auth_config)
    elif is_access_token_login(auth_config, anonymous):
        connection_validation_result = __validate_server_connection(
            auth_config, BearerAuth(auth_config.access_token)
        )
    elif is_anonymous_login(auth_config, anonymous):
        connection_validation_result = __validate_server_connection(
            auth_config, EmptyAuth()
        )
    else:
        connection_validation_result = False

    if connection_validation_result:
        save_auth_config(auth_config)
    return connection_validation_result


def __login_by_username_password(auth_config: AuthConfig) -> bool:
    if auth_config.user is not None and auth_config.password is not None:
        auth_token = HTTPBasicAuth(auth_config.user, auth_config.password)
        connection_validation_result = __validate_server_connection(
            auth_config, auth_token
        )
        if connection_validation_result:
            encrypted_password = get_encrypted_password(auth_config, auth_token)
            if encrypted_password is not None:
                auth_config.password = encrypted_password
                return True
    return False


def __validate_server_connection(auth_config: AuthConfig, auth_token: AuthBase) -> bool:
    success = False
    try:
        logger.debug("Attempting to ping artifactory")
        response = ArtifactoryApi(auth_config.artifactory_url, auth_token).ping()
        if response.status_code == http.HTTPStatus.OK:
            success = True
        else:
            logger.debug(
                f"Expected {http.HTTPStatus.OK} status but got {response.status_code} "
                f"when using url {auth_config.artifactory_url}"
            )
    except requests.exceptions.ConnectionError as e:
        logger.debug(f"Unable to connect to the provided url :{e}.")
    except requests.exceptions.MissingSchema as e:
        logger.debug(f"Invalid Artifactory URL provided: {e}.")
    except requests.exceptions.RequestException as e:
        logger.debug(f"Unexpected request exception: {e}.")

    return success



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/kube_deployment_captain/traffic_mapping.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2qwak/kube_deployment_captain/traffic_mapping.proto\x12\x1cqwak.kube.deployment.captain\"\xd8\x01\n\x10VariationMapping\x12\'\n\x1fselected_variation_mapping_name\x18\x01 \x01(\t\x12\"\n\x1atraffic_request_timeout_ms\x18\x04 \x01(\x05\x12\x15\n\rdeployment_id\x18\x03 \x01(\t\x12P\n\x14traffic_mapping_spec\x18\x02 \x01(\x0b\x32\x30.qwak.kube.deployment.captain.TrafficMappingSpecH\x00\x42\x0e\n\x0cmapping_spec\";\n\x12TrafficMappingSpec\x12\x12\n\npercentage\x18\x01 \x01(\x05\x12\x11\n\tis_shadow\x18\x02 \x01(\x08\x42+\n\'com.qwak.ai.kube.deployment.captain.apiP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.kube_deployment_captain.traffic_mapping_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\'com.qwak.ai.kube.deployment.captain.apiP\001'
  _globals['_VARIATIONMAPPING']._serialized_start=85
  _globals['_VARIATIONMAPPING']._serialized_end=301
  _globals['_TRAFFICMAPPINGSPEC']._serialized_start=303
  _globals['_TRAFFICMAPPINGSPEC']._serialized_end=362
# @@protoc_insertion_point(module_scope)
